<?php

/**
 * Fonctions utiles
 * @Version 02-2020
 * @Projet SambaEdu
 * Fonctions Interface Homme/Machine
 * @Auteurs Denis Bonnenfant @ Equipe Sambaedu
 * @Note: Ce fichier de fonction doit etre appele par un include
 * @Licence Distribue sous la licence GPL
 */

/**
 *
 * file: fonc_outils.inc.php
 *
 * @Repertoire: includes/
 */
require_once "logs.inc.php";

function convertWindowsTimestamp($wintime)
{
    return $wintime / 10000000 - 11644477200;
}

/**
 *
 * Fonctions qui retourne le nom d'une machine en fonction de son adresse IP
 *
 * @Parametres Adresse IP  de la machine
 *
 * @return Le nom de la machine
 *        
 */
function avoir_nom($ipHost)
{ // Retourne le nom d'une machine a partir de l'adresse IP ou 0 si pas
    $mp_curr = search_ad("(&(ipHostNumber=$ipHost)(objectClass=ipHost))", "computers");
    if (isset($mp_curr[0]["cn"])) {
        $mpenc = $mp_curr[0]['cn'];
        return $mpenc;
    } else {
        return 0;
    }
}

/**
 * Demarre, eteint ou reboote un poste
 *
 * @Parametres action: wol, reboot, shutdown
 *             nom : nom du poste
 *
 * @return
 *
 */
function start_poste($config, $action, $name, $force = false, $silent = false)
{
    $name = strtolower($name);

    if ($name == $config['se4fs_name'] || $name == $config['se4ad_name'])
        return "Ca va pas ?";

    $machine = search_machine($config, $name, true);
    $ret = false;
    $out = array();
    $msg = "";
    $status = get_machine_status($config, $name);
    $ip = $machine['iphostnumber'] ?? $name;
    $ping = fping($ip);
    switch ($action) {
        case "start":
        case "wol":
            if (! $ping || $status['state'] == "off" || $force) {
                if (empty($machine['networkaddress'])) {
                    log_connexion($config, "", $name, "wol", "startup", "", SAMBAEDU_NO_WOL);
                    $msg .= "On n'allume pas  <b>$name</b> : elle n'a pas d'adresse mac enregistrée !";
                    break;
                }
                $reseau = get_vlan($config, @$machine['iphostnumber']);
                $msg .= "Mise en marche de la machine  <b>$name</b> : ";
                exec("/usr/bin/wakeonlan -i " . long2ip($reseau['broadcast']) . " " . $machine['networkaddress'], $out);
                $msg .= end($out) . "<br>";
                if (isset($config['wol_broadcast'])) {
                    exec("/usr/bin/wakeonlan -i " . $config['wol_broadcast'] . " " . $machine['networkaddress'], $out);
                    $msg .= end($out) . "<br>";
                }
                if (! $ping) {
                    log_connexion($config, "", $name, "wol", "startup");
                }
            } else {
                $msg .= "On n'allume pas  <b>$name</b> : elle est déjà allumée !";
            }
            break;

        case "reboot":
            if ($ping && ($status['state'] != "off" || $force)) {
                $msg .= "On reboote avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>\n";
                // if ($status['state'] != "linux") {
                if ($ping == "windows") {
                    // machine windows
                    if ($silent) {
                        $commande = "/usr/bin/net rpc shutdown -k -t 2 -f -r -C 'Reboot demande par le serveur sambaEdu' -U www-admin% -S " . $name . " >/dev/null 2>&1 &";
                    } else {
                        $commande = "/usr/bin/net rpc shutdown -k -t 2 -f -r -C 'Reboot demande par le serveur sambaEdu' -U www-admin% -S " . $name . " 2>&1";
                    }
                    exec($commande, $out, $ret);
                    if ($ret != 0) {
                        $msg .= "<b>Attention, reboot impossible</b> : " . end($out) . "<br><br>";
                        log_connexion($config, "", $name, "windows", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= start_poste($config, "wol", $name, true);
                    } else {
                        log_connexion($config, "", $name, "windows", "shutdown");
                        $msg .= end($out);
                    }
                    $msg .= "<br><br>";
                } elseif ($ping == "linux") {
                    // poste linux : ssh...
                    exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine['cn'] . " shutdown -r now", $out, $ret);
                    $msg .= end($out) . "<br><br>";
                    if ($ret != 0) {
                        $msg .= "On reboote avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>\n";
                        $msg .= "<b>Attention, reboot impossible</b>, on essaye de la démarrer <br><br>";
                        log_connexion($config, "", $name, "linux", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= start_poste($config, "wol", $name, true);
                    } else {
                        log_connexion($config, "", $name, "linux", "shutdown");
                        $msg .= "OK<br>\n";
                    }
                }
            } else {
                $msg .= "<b>$name</b>  est éteinte : on l'allume : <br>\n";
                $msg .= start_poste($config, "wol", $name, true);
            }
            break;

        case "stop":
        case "shutdown":
            if ($force || @count($status['user']) == 0) {
                if ($ping) {
                    $msg .= "On éteint avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>\n";
                }
                // if ($status['state'] != "linux") {
                if ($ping == "windows") {

                    // machine windows
                    if ($silent) {
                        $commande = "/usr/bin/net rpc shutdown -k -t 30 -f -C \"Arret demande par le serveur sambaEdu\" -U www-admin% -S " . $name . " >/dev/null 2>&1 &";
                    } else {
                        $commande = "/usr/bin/net rpc shutdown -k -t 30 -f -C \"Arret demande par le serveur sambaEdu\" -U www-admin% -S " . $name . " 2>&1";
                    }
                    exec($commande, $out, $ret);
                    if ($ret != 0) {
                        $msg .= "<b>Attention, arrêt impossible</b> : " . end($out) . "<br><br>";
                        log_connexion($config, "", $name, "windows", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                    } else {
                        log_connexion($config, "", $name, "windows", "shutdown");
                        $msg .= end($out);
                    }
                    $msg .= "<br><br>";
                } elseif ($ping == "linux") {
                    // poste linux : ssh...
                    exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine['cn'] . " shutdown -h now", $out, $ret);
                    $msg .= end($out) . "<br><br>";
                    if ($ret != 0) {
                        log_connexion($config, "", $name, "linux", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= "On &#233;teint avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>\n";
                        $msg .= "<b>Attention, arrêt impossible</b>, la machine est injoignable ! <br><br>";
                    } else {
                        log_connexion($config, "", $name, "linux", "shutdown");
                        $msg .= "OK<br>\n";
                    }
                }
            } else {
                $msg .= "On n'éteint pas le <b> le poste <b>" . $name . "</b> : un utilisateur est connecté<br>\n";
            }
            break;
    }
    return $msg;
}

/**
 * Demarre, eteint ou reboote un parc
 *
 * @Parametres action: wol, reboot, shutdown
 *             parc : nom du parc
 *
 * @return
 *
 */
function start_parc($config, $action, $parc, $force = false, $silent = true)
{
    $liste = list_members_parc($config, $parc, false);
    sort($liste);
    $ret = "";
    foreach ($liste as $value) {
        $ret .= start_poste($config, $action, $value, $force, $silent) . "<br>\n";
        // usleep(300000);
    }
    return $ret;
}

/**
 * Affiche la date dans le format j/m/a
 *
 * @Parametres la date
 *
 * @return la date au format j/m/a
 */
function affiche_date($date)
{
    list ($a, $m, $j) = preg_split("/-/", $date);
    return "$j/$m/$a";
}

/**
 * Retourne une liste sous forme d'un tableau
 *
 * @Parametres $liste
 *
 * @return la tableau
 */
function liste_tab($liste)
{
    $t = preg_split("/\|/", $liste);
    for ($i = 0; $i < count($t); $i = $i + 2) {
        $cle = $t[$i];
        $val = $t[$i + 1];
        $tab[$cle] = $val;
    }
    return $tab;
}

/**
 * Retourne un select pour choisir la date
 *
 * @Parametres date et param
 * @return string select au sens HTML
 *        
 */
function choix_date($date, $param)
{
    $tab_mois = array(
        9 => gettext("Septembre"),
        10 => gettext("Octobre"),
        11 => gettext("Novembre"),
        12 => gettext("D&#233;cembre"),
        1 => gettext("Janvier"),
        2 => gettext("F&#233;vrier"),
        3 => gettext("Mars"),
        4 => gettext("Avril"),
        5 => gettext("Mai"),
        6 => gettext("Juin"),
        7 => gettext("Juillet"),
        8 => gettext("Ao&#251;t")
    );
    list ($an, $mois, $jour) = preg_split("/-/", $date);
    $html = "<select name=\"jour_$param\">";
    // <option value=\"jour\"> jour</option>";
    for ($i = 1; $i <= 31; $i ++) {
        $ii = ($i < 10 ? "0$i" : $i);
        $html .= "<option value=\"$ii\" " . ($ii == $jour ? "selected" : "") . "> $ii</option>";
    }
    $html .= "</select>";
    $html .= "<select name=\"mois_$param\"> ";
    // <option value=\"mois\"> mois</option>";
    for ($i = 1; $i <= 12; $i ++) {
        $ii = ($i < 10 ? "0$i" : $i);
        $html .= "<option value=\"$ii\" " . ($ii == $mois ? "selected" : "") . "> $tab_mois[$i]</option>";
    }
    $html .= "</select>";
    $html .= "<select name=\"an_$param\"> ";
    // <option value=\"an\"> ann&#233;e</option>";
    $html .= "<option value=\"$an\" selected> $an</option>";
    $an_suivant = $an + 1;
    $html .= "<option value=\"$an_suivant\" > $an_suivant</option>";
    $html .= "</select>";
    return $html;
}

/**
 * a partir d'un uid d'élève fournit le nom inversé de son répertoire classe
 * en cas de login prenom.nom, renvoie nom.prenom, sinon renvoie le login
 *
 * @Parametres $uid
 *
 * @return $rep
 */
function inverse_login($login)
{
    $tab = preg_split('/\./', $login);
    if (count($tab) == 2) {
        $rep = $tab[1] . "." . $tab[0];
    } else {
        $rep = $login;
    }
    return $rep;
}

function affiche_all_groups($config, $align, $afftype, $cours_gr = [])
{
    // // Etablissement des listes des groupes disponibles
    $list_groups = search_ad($config, "*", "group", $config['dn']['groups']);
    echo $config['dn']['groups'];
    // Etablissement des sous listes de groupes :
    $j = 0;
    $k = 0;
    $l = 0;
    $i = 0;
    $classe = [];
    $cours = $matiere = $equipe = $autres = array();
    for ($loop = 0; $loop < count($list_groups); $loop ++) {
        // Cours
        if (preg_match("/Cours_/", $list_groups[$loop]["cn"])) {
            $cours[$i]["cn"] = $list_groups[$loop]["cn"];
            $cours[$i]["description"] = $list_groups[$loop]["samaccountname"] ?? "";
            $i ++;
        }
        // // Classe
        if (preg_match("/Classe_/", $list_groups[$loop]["cn"])) {
            $classe[$j]["cn"] = $list_groups[$loop]["cn"];
            $classe[$j]["description"] = $list_groups[$loop]["samaccountname"] ?? "";
            $j ++;
        } // Matiere
        elseif (preg_match("/Matiere_/", $list_groups[$loop]["cn"])) {
            $matiere[$l]["cn"] = $list_groups[$loop]["cn"];
            $matiere[$l]["description"] = $list_groups[$loop]["samaccountname"] ?? "";
            $l ++;
        } // Equipe
        elseif (preg_match("/Equipe_/", $list_groups[$loop]["cn"])) {
            $equipe[$k]["cn"] = $list_groups[$loop]["cn"];
            $equipe[$k]["description"] = $list_groups[$loop]["samaccountname"] ?? "";
            $k ++;
        } // Autres
          // elseif (! preg_match("/^overfill/", $list_groups[$loop]["cn"]) && ! preg_match("/^lcs-users/", $list_groups[$loop]["cn"]) && ! preg_match("/nogroup/", $list_groups[$loop]["cn"]) && ! preg_match("/Cours_/", $list_groups[$loop]["cn"]) && ! preg_match("/Matiere_/", $list_groups[$loop]["cn"]) && ! preg_match("/root/", $list_groups[$loop]["cn"]) && ! preg_match("/invites/", $list_groups[$loop]["cn"]) && ! preg_match("/^machines/", $list_groups[$loop]["cn"])) {
          // $autres[$m]["cn"] = $list_groups[$loop]["cn"];
          // $autres[$m]["description"] = $list_groups[$loop]["description"] ?? "";
          // $m ++;
          // }
    }

    // Affichage des boites de s&#233;lection des groupes sur lesquels fixer les quotas + choix d'un user sp&#233;cifique
    echo "
		<table align=\"$align\" border=\"0\" cellspacing=\"10\" style='float: none' >
		<tr>
		<td>" . gettext("Classes") . "</td>";
    echo "<td>" . gettext("Mati&#232res") . "</td>";
    if ($afftype == "cours")
        echo "<td>" . gettext("Cours") . "</td>";
    echo "<td>" . gettext("Equipes") . "</td>
		<td>" . gettext("Autres") . "</td>";
    if ($afftype == "user")
        echo "<td>" . gettext("Utilisateur sp&#233;cifique") . "</td>";
    echo "</tr>
		<tr>
		<td valign=\"top\">
        
		<select name= \"classe_gr[]\" size=\"8\" multiple=\"multiple\">\n";
    for ($loop = 0; $loop < count($classe); $loop ++) {
        echo "<option value=" . $classe[$loop]["cn"] . ">" . $classe[$loop]["description"];
    }
    echo "</select>";
    echo "</td>\n";

    echo "<td valign=\"top\">\n";
    echo "<select name= \"matiere_gr[]\" size=\"8\" multiple=\"multiple\">\n";
    for ($loop = 0; $loop < count($matiere); $loop ++) {
        echo "<option value=" . $matiere[$loop]["cn"] . ">" . $matiere[$loop]["description"];
    }
    echo "</select></td>\n";
    // echo "<td valign=\"top\">";

    if ($afftype == "cours") {
        echo "<td valign=\"top\">\n";
        echo "<select name= \"cours_gr[]\" value=\"$cours_gr\" size=\"10\" multiple=\"multiple\">";
        for ($loop = 0; $loop < count($cours); $loop ++) {
            echo "<option value=" . $cours[$loop]["cn"] . ">" . $cours[$loop]["description"];
        }
        echo "</select>";
        echo "</td>";
    }

    echo "<td valign=\"top\">\n";
    echo "<select name= \"equipe_gr[]\" size=\"8\" multiple=\"multiple\">\n";
    for ($loop = 0; $loop < count($equipe); $loop ++) {
        echo "<option value=" . $equipe[$loop]["cn"] . ">" . $equipe[$loop]["description"];
    }
    echo "</select></td>\n";
    echo "<td valign=\"top\">";
    $autres = filter_group_autres($config);
    $autres_projets = filter_group_projets($config);
    echo "<select name=\"autres_gr[]\" size=\"8\" multiple=\"multiple\">";
    echo "<option value=administratifs>administratifs";
    echo "<option value=eleves>eleves";
    echo "<option value=profs>profs";
    for ($loop = 0; $loop < count($autres); $loop ++) {
        echo "<option value=" . $autres[$loop]["cn"] . ">" . $autres[$loop]["cn"];
    }
    for ($loop = 0; $loop < count($autres_projets); $loop ++) {
        echo "<option value=" . $autres_projets[$loop]["cn"] . ">" . $autres_projets[$loop]["cn"];
    }
    echo "</select></td>\n";
    if ($afftype == "user")
        echo "<td valign=\"top\"><INPUT TYPE=\"TEXT\" NAME=\"user\" size=15></td>\n";
    echo "</tr><tr></tr></table>\n\n";
}

?>
