<?php

/**
 * Fonctions LDAP
 *
 * @Projet LCS / SambaEdu
 *
 * @Auteurs Equipe Sambaedu
 *
 * @version $Id: ldap.inc.php 09-11-2018 mrfi $
 *         
 * @Note: Ce fichier de fonction doit etre appele par un include
 *
 * @Licence Distribue sous la licence GPL
 */
/**
 *
 * file: ldap.inc.php
 *
 * @Repertoire: includes/
 */
require_once ("lang.inc.php");
bindtextdomain('sambaedu-core', "/var/www/sambaedu/locale");
textdomain('sambaedu-core');

// pour utiliser bind_ad_gssapi
// include_once "config.inc.php";
include_once "functions.inc.php";
require_once "samba-tool.inc.php";
require_once "dhcpd.inc.php";
require_once "fonc_parc.inc.php";

// Pour activer/desactiver la modification du givenname (Prenom) lors de la modification dans annu/mod_user_entry.php
$corriger_givenname_si_diff = "n";

function lock($var)
{
    $cpt = 0;
    while (! apcu_add($var . '_lock', true, 5)) {
        usleep(10000);
        if ($cpt ++ > 1000) {
            exit();
        }
    }
}

function unlock($var)
{
    apcu_delete($var . '_lock');
}

// fonctions validées se4
function array_unique_multi($input)
{
    $serialized = array_map('serialize', $input);
    $unique = array_unique($serialized);
    return array_intersect_key($input, $unique);
}

/**
 * Comparaison sur fullname
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier le fullname
 * @type
 * @Parametres $a - La premiere entree 	$b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 */
function cmp_fullname($a, $b)
{
    return strcmp($a["fullname"], $b["fullname"]);
}

/**
 * Comparaison sur name
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier le name
 * @type
 * @Parametres $a - La premiere entree 	$b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 *
 */
function cmp_nom($a, $b)
{
    $a['nom'] = $a['nom'] ?? "";
    $b['nom'] = $b['nom'] ?? "";
    return strcmp($a["nom"], $b["nom"]);
}

/**
 * Comparaison sur cn
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier le cn (common name)
 * @Parametres  $a - La premiere entree  $b - La deuxieme entree a comparer
 * @type
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 */
function cmp_cn($a, $b)
{
    return strcmp($a["cn"], $b["cn"]);
}

/**
 * Comparaison sur groupe
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier les groupes
 * @type
 * @Parametres  $a - La premiere entree 	$b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 *
 */
function cmp_group($a, $b)
{
    return strcmp($a["group"], $b["group"]);
}

/**
 * Comparaison sur categories
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier les categories
 * @type
 * @Parametres  $a - La premiere entree  $b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 */
function cmp_cat($a, $b)
{
    return strcmp($a["cat"], $b["cat"]);
}

/**
 * commparaison sur printer-name
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier le printer-name, insensible a la case
 * @type
 * @Parametres  $a - La premiere entree  $b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 */
function cmp_printer($a, $b)
{
    return strcasecmp($a["name"], $b["name"]);
}

/**
 * commparaison sur printer-location
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier le printer-location, insensible a la case
 * @type
 * @Parametres  $a - La premiere entree  $b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 */
function cmp_location($a, $b)
{
    return strcasecmp($a["location"], $b["location"]);
}

function sort_by_key($key, &$array)
{
    array_multisort(array_column($array, $key), SORT_ASC, $array);
}

/**
 * dn vers cn
 *
 * @description permet de passer du format dn complet au format cn
 * @type
 * @Parametres string $dn
 * @returns string $cn
 */
function ldap_dn2cn_case(string $dn)
{
    $rdns = explode(",", $dn);
    $cn = explode("=", $rdns[0]);
    return $cn[1];
}

/**
 * dn vers cn
 *
 * @description permet de passer du format dn complet au format cn
 * @type
 * @Parametres string $dn
 * @returns string $cn
 */
function ldap_dn2cn(string $dn)
{
    $rdns = explode("=", explode(',', $dn)[0]);
    return $rdns[1] ?? $dn;
}

/**
 *
 * @description permet de passer du format dn complet au format ou=truc
 * @type
 * @Parametres string $dn
 * @returns string $cn
 */
function ldap_dn2ou(string $dn)
{
    $rdns = explode(",", $dn, 2);
    $ou = $rdns[0];
    return $ou;
}

/**
 * retourne le dn de l'ou parente
 *
 * @param string $dn
 * @return mixed
 */
function ldap_dn2oudn(string $dn)
{
    $rdns = explode(",", $dn, 2);
    return $rdns[1];
}

/**
 * retourne le pseudo-samaccountname de l'ou parente
 *
 * @param string $dn
 * @return string $parent
 */
function ldap_dn2parent(string $dn)
{
    $parent = ldap_dn2cn(ldap_dn2oudn($dn));
    if (! preg_match("/^[0-9]{7}[a-z]/i", $parent) && ! empty($s = substr(ldap_dn2uai($dn), 3)) && ! preg_match("/" . $parent . "/i", "Computers")) {
        $parent = $parent . "-" . $s;
    }
    return $parent;
}

/**
 * retourne le samaccountname du groupe parc
 *
 * @param string $dn
 * @return string $samaccountname
 */
function ldap_dn2sam(string $dn)
{
    $sam = ldap_dn2cn($dn);
    if (! preg_match("/^[0-9]{7}[a-z]/i", $sam) && ! empty($s = substr(ldap_dn2uai($dn), 3))) {
        $sam = $sam . "-" . $s;
    }
    return $sam;
}

/**
 * retourne le nom à partir du sam (multi-etab)
 *
 * @param string $sam
 * @return string $cn
 */
function ldap_sam2cn(string $sam)
{
    if (preg_match("/^(.*)-([0-9]{4}[a-z])$/i", $sam, $m)) {
        $sam = $m[1];
    }
    return $sam;
}

/**
 * retourne le samaccountname du groupe parc
 *
 * @param string $dn
 * @return string $samaccountname
 */
function ldap_sam2suffix(string $sam)
{
    if (preg_match("/^(.*)(-[0-9]{4}[a-z])$/i", $sam, $m)) {
        $suffix = $m[2];
    } else {
        $suffix = "";
    }
    return $suffix;
}

/**
 * retourne l'uai à partir du dn
 *
 * @param string $dn
 * @return mixed
 */
function ldap_dn2uai(string $dn)
{
    $rdn = preg_split("/,dc=/i", $dn);
    if (preg_match("/(ou=|cn=|-)([0-9]{7}[a-z]),/i", $rdn[0], $m)) {
        return strtolower($m[2]);
    } else {
        return 0;
    }
}

/**
 * formate le samaccountname pour une machine en le mettant en minuscule et sans le $ final
 *
 * @param array $machine
 * @return mixed
 */
function ldap_hostname(array $config, $machine)
{
    $hostname = $machine['samaccountname'] ?? $machine['cn'] ?? $machine ?? "";
    // return strtolower(preg_replace("/(" . $config['suffix'] . ")?\\$?/", "", $hostname));
    return str_replace("$", "", strtolower($hostname));
}

function add_hostname_suffix(array $config, $name)
{
    if (! preg_match("/" . $config['suffix'] . "/i", $name)) {
        $name .= $config['suffix'];
    }
    return strtolower($name);
}

function get_uai_from_hostname(array $config, $name)
{
    if (preg_match("/-([0-9]{4}[a-z])/i", $name, $m)) {
        if (! empty($config['departement'])) {
            $uai = strtolower($config['departement'] . $m[1]);
        } elseif (! empty($config['etab_ou'])) {
            $uai = strtolower(substr($config['etab_ou'], 0, 3) . $m[1]);
        } else {
            $uai = 0;
        }
    } elseif (! empty($config['etab_ou'])) {
        $uai = strtolower($config['etab_ou']);
    } else {
        $uai = 0;
    }
    return $uai;
}

function remove_hostname_suffix(array $config, $name)
{
    $name = preg_replace("/" . $config['suffix'] . "/i", "", $name);
    return strtolower($name);
}

function to_guid($guid)
{
    $hex_guid = unpack("H*hex", $guid);
    $hex = $hex_guid["hex"];

    $hex1 = substr($hex, - 26, 2) . substr($hex, - 28, 2) . substr($hex, - 30, 2) . substr($hex, - 32, 2);
    $hex2 = substr($hex, - 22, 2) . substr($hex, - 24, 2);
    $hex3 = substr($hex, - 18, 2) . substr($hex, - 20, 2);
    $hex4 = substr($hex, - 16, 4);
    $hex5 = substr($hex, - 12, 12);

    $guid = $hex1 . "-" . $hex2 . "-" . $hex3 . "-" . $hex4 . "-" . $hex5;

    return $guid;
}

function from_guid($uuid)
{
    return call_user_func_array('pack', array_merge(array(
        'VvvCCC6'
    ), array_map('hexdec', array(
        substr($uuid, 0, 8),
        substr($uuid, 9, 4),
        substr($uuid, 14, 4),
        substr($uuid, 19, 2),
        substr($uuid, 21, 2)
    )), array_map('hexdec', str_split(substr($uuid, 24, 12), 2))));
}

/**
 *
 * Fonction de generation de mot de passe faciles à retenir
 * génère dans tous les cas un mot de passe compatible avec Windows complexity=on
 *
 * @return string $password
 *        
 */
function create_random_password(int $nb_chars = 8, bool $complex = false, $easy = true)
{
    if ($easy) {
        // Liste de mots, pensez à choisir des mots avec des sons qui se prononcent
        // facilement !

        $mots = array(
            "bleu",
            "plus",
            "moins",
            "point",
            "virgule",
            "as",
            "coeur",
            "carreau",
            "trefle",
            "pique",
            "blanc",
            "rouge",
            "jaune",
            "vert",
            "valet",
            "pizza",
            "chaine",
            "genre",
            "cafe",
            "fonction",
            "eponge",
            "lapin",
            "renard",
            "image",
            "gateau",
            "pied",
            "hasard",
            "source",
            "chat",
            "souris",
            "chapeau",
            "langue",
            "arbre",
            "bonbon",
            "livre",
            "velo",
            "tout",
            "vecteur",
            "piano",
            "vacances",
            "flute",
            "fuite",
            "zebre",
            "zorro",
            "loutre",
            "deux",
            "trois",
            "quatre",
            "cinq",
            "sept",
            "chien",
            "chat",
            "poulet",
            "koala",
            "huit",
            "neuf",
            "douze",
            "treize",
            "un",
            "do",
            "re",
            "mi",
            "fa",
            "sol",
            "la",
            "si"
        );
    } else {
        $mots = [];
        $lettres = "azertyuipqsdfghjkmwxcvbn";
        for ($i = 0; $i < 100; $i ++) {
            $mot = "";
            $long = rand(4, 7);
            for ($j = 0; $j < $long; $j ++) {
                $mot .= substr($lettres, rand(0, strlen($lettres) - 1), 1);
            }
            $mots[$i] = $mot;
        }
        // return print_r($mots);
    }
    $specials = array(
        "+",
        "-",
        "*",
        "$",
        "!"
    );
    $map = [
        "i" => 1,
        "o" => 0,
        "s" => 5,
        "e" => 3
    ];

    $m1 = $mots[rand(0, count($mots) - 1)];
    while (strlen($m1) > $nb_chars - 3) {
        $m1 = $mots[rand(0, count($mots) - 1)];
    }
    $r1 = $nb_chars - strlen($m1) - 1;
    $m2 = $mots[rand(0, count($mots) - 1)];
    $i = 0;
    while ($i ++ < 100 && (strlen($m2) > $r1 || (strlen($m2) > $r1 - 3 && strlen($m2) < $r1))) {
        $m2 = $mots[rand(0, count($mots) - 1)];
    }
    $r2 = $r1 - strlen($m2) - 1;
    if ($complex) {
        $s = $specials[rand(0, 4)];
    } else {
        $s = "-";
    }
    if ($r2 > 1) {
        $m3 = $mots[rand(0, count($mots) - 1)];
        $i = 0;
        while (strlen($m3) != $r2 && $i < 100) {
            $m3 = $mots[rand(0, count($mots) - 1)];
            $i ++;
        }
        $r = ucfirst($m1) . $s . ucfirst($m2) . "-" . ucfirst($m3);
    } else {
        $r = ucfirst($m1) . $s . ucfirst($m2);
    }

    if ($complex) {
        foreach ($map as $l => $c) {
            $res = preg_replace("/" . $l . "/", $c, $r, 1);
            if ($res != $r)
                break;
        }
    } else {
        $res = $r;
    }
    return $res;
}

/**
 * A définir !
 *
 * @description todo
 * @type
 * @Parametres
 * @returns
 */
function remove_count($arr)
{
    foreach ($arr as $key => $val) {
        if ($key === "count")
            unset($arr[$key]);
        elseif (is_array($val))
            $arr[$key] = remove_count($arr[$key]);
    }
    return $arr;
}

function check_ad($config, $uai = false)
{
    // verification de l'arborescence AD
    if ($uai == 0) {
        $uai = false;
    }
    if ($uai || empty($config['ad_check'])) {
        $config_local = $config;
        $config = get_config($config, true, $uai);
        foreach ($config['dn'] as $branche) {
            ouadd($config, $branche, "");
        }
        $config = $config_local;
        unset($config_local);
    }
    if (empty($config['ad_check'])) {
        $config_local = $config;
        $config = get_config($config, true, true);
        if (! empty($config['etab_ou'])) {
            // groupes globaux définis à l'échelle du domaine
            // ouadd($config, "OU=" . $config['etab_ou'] . "," . $config['ldap_base_dn'], "", "dn");
            $admin = search_user($config, "admin");
            if (ldap_dn2ou($admin['dn']) == $config['admin_rdn']) {
                move_ad($config, "admin", "cn=admin," . $config['dn']['people'], "user");
            }
            groupadd($config, "Cloud", $config['groups_rdn'], "groupe des utilisateurs cloud");
            groupadd($config, "Cloud_etab", $config['groups_rdn'], "groupe des utilisateurs migrés vers cloud etablissement");
            groupadd($config, "Cloud_nuage.apps", $config['groups_rdn'], "groupe des utilisateurs vers cloud nuage.apps");
            groupadd($config, "Eleves", $config['groups_rdn'], "groupe des eleves");
            groupadd($config, "Profs", $config['groups_rdn'], "groupe des profs");
            groupadd($config, "Administratifs", $config['groups_rdn'], "groupe des administratifs");
            groupadd($config, "Direction", $config['groups_rdn'], "groupe des personnels de direction");
            groupadd($config, "VieScol", $config['groups_rdn'], "groupe des personnels de vie scolaire");
            groupadd($config, "Medical", $config['groups_rdn'], "groupe des personnels médicaux");
            groupadd($config, "Agent", $config['groups_rdn'], "groupe des personnels collectivité (Dpt/Région)");
            groupadd($config, "Secretariat", $config['groups_rdn'], "groupe des personnels secretatiat administratif");
            groupadd($config, "Documentaliste", $config['groups_rdn'], "groupe des enseignants documentalistes");
            groupadd($config, "computer_is_admin", $config['rights_rdn'], "administration des ordinateurs");
            // groupadd($config, "", $config['parcs_rdn'], "administration des ordinateurs");
            if (! have_right($config, "se3_is_admin", "admin")) {
                add_right($config, "admin", "se3_is_admin");
            }
            // retour dans le contexte de l'etablissement
        } else {
            $admin = search_user($config, "admin");
            if (ldap_dn2ou($admin['dn']) == $config['admin_rdn']) {
                move_ad($config, "admin", "cn=admin," . $config['dn']['people'], "user");
            }
            groupadd($config, "Cloud", $config['groups_rdn'], "groupe des utilisateurs cloud");
            groupadd($config, "Cloud_etab", $config['groups_rdn'], "groupe des utilisateurs migrés vers cloud etablissement");
            groupadd($config, "Cloud_nuage.apps", $config['groups_rdn'], "groupe des utilisateurs migrés vers cloud nuage.apps");
            groupadd($config, "Eleves", $config['groups_rdn'], "groupe des eleves");
            groupadd($config, "Profs", $config['groups_rdn'], "groupe des profs");
            groupadd($config, "Administratifs", $config['groups_rdn'], "groupe des administratifs");
            groupadd($config, "Direction", $config['groups_rdn'], "groupe des personnels de direction");
            groupadd($config, "VieScol", $config['groups_rdn'], "groupe des personnels de vie scolaire");
            groupadd($config, "Medical", $config['groups_rdn'], "groupe des personnels médicaux");
            groupadd($config, "Agent", $config['groups_rdn'], "groupe des personnels collectivité (Dpt/Région)");
            groupadd($config, "Secretariat", $config['groups_rdn'], "groupe des personnels secretatiat administratif");
            groupadd($config, "Documentaliste", $config['groups_rdn'], "groupe des enseignants documentalistes");
            groupadd($config, "computer_is_admin", $config['rights_rdn'], "administration des ordinateurs");
            // groupadd($config, "", $config['parcs_rdn'], "administration des ordinateurs");
            if (! have_right($config, "se3_is_admin", "admin")) {
                add_right($config, "admin", "se3_is_admin");
            }
        }
        $config = $config_local;
        groupadd($config, "password_is_admin", $config['rights_rdn'], "gestion des mots de passe");
        set_config($config, 'ad_check', 1, 'sambaedu');
    }
    return $config;
}

function test_ad($config)
{
    $time = microtime(true);
    $i = 0;
    while (microtime(true) - $time <= 10) {
        apcu_store("ldap_cache_invalid", true, 10);
        search_user($config, "admin");
        $i ++;
    }
    return $i;
}

/**
 * Recherche des objets dans l'AD
 *
 * @description Recherche des objets dans AD en fonction du type demandé
 * @type tableau
 * @Parametres $name - le nom de l'objet (cn|ou|dn) "*" pour tout
 * $type - le type d'objets à chercher : user, employeenumber, filter, member, memberof, pp, dn (default), machine, salle, parc, group, classe, equipe, matiere, projet, cours, autre, imprimante, delegation, rights, type, gpo
 * $branch - la branche de recherche "all" pour tout (sans effet pour dn)
 * @returns Retourne un tableau avec les objets et leurs attributs utiles
 *
 * les resultats sont cachés 60 s, le cache est invalidé par toute opération d'écriture dans l'AD dont php a connaissance. Cela n'inclut pas les opérations faites directement sur l'AD par des clients externes.
 */
function search_ad(array $config, string $name, string $type = "dn", string $branch = "all", array $attrs = array(), string $scope = "subtree")
{
    $hash = md5($name . $type . $branch . serialize($attrs) . $scope);
    if (apcu_fetch("ldap_cache_invalid")) {
        apcu_delete(new APCUIterator('#^search_#'));
        apcu_delete(new APCUIterator('#^rights_#'));
        apcu_delete("ldap_cache_invalid");
    } elseif ($result = apcu_fetch("search_" . $hash)) {
        return $result;
    }
    // Initialisation
    $info = array();
    // correspondance des attributs: ldapAD => php uniquement pour les utilisateurs !
    $map = array(
        "sn" => "nom",
        "displayname" => "fullname",
        "givenname" => "prenom",
        "initials" => "pseudo",
        "mail" => "email",
        "telephonenumber" => "tel"
    );

    // LDAP attributs
    switch ($type) {
        case "user":
            $filter = "(&(objectclass=user)(cn=" . $name . "))";
            $ldap_attrs = array(
                "cn", // login
                "displayname", // Prenom Nom
                "sn", // Nom
                "givenname", // Pseudo -> Prenom
                "mail", // Mail
                "telephonenumber", // Num telephone
                "description",
                "physicaldeliveryofficename", // Date de naissance,Sexe (F/M) hash
                "title", // numero unique id ENT (OpenENT),xternalId
                "employeenumber", // identifiants unique SIECLE et/ou GPEI, ASM... (séparés par des ,)
                "initials", // pseudo
                "useraccountcontrol", // état du compte actif : 512, desactivé 514
                "memberof", // groupes
                "userprincipalname", // pseudo-adresse mail correspondant au login ENT
                "objectguid", // identifiant unique à décoder avec to_guid()
                "pwdlastset"
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "employeenumber": // recherche par le n° Siecle/STS
        case "id": // recherche par le n° Siecle/STS
            $filter = "(&(objectclass=user)(|(title=" . $name . ")(title=" . sprintf("%05d", $name) . ")(title=" . preg_replace("/^0*/", "", $name) . ")))";
            $ldap_attrs = array(
                "cn" // login
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "userprincipalname": // recherche par l'identifiant ENT
            $filter = "(&(objectclass=user)(|(cn=" . $name . ")(userprincipalname=" . $name . "@" . $config['ent_domain'] . ")(userprincipalname=" . $name . "@" . $config['domain'] . ")))";
            $ldap_attrs = array(
                "cn" // login
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;

        case "filter": // user ou group
            $filter = $name;
            $ldap_attrs = array(
                "cn",
                "displayname",
                "description" //
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "member": // cherche si user ou group est dans la ou $branch (profs eleves administratifs)
            $filter = "(&(cn=" . $name . ")(objectclass=user))";
            $ldap_attrs = array(
                "cn" // login
            );
            $branch = "ou=" . $branch . "," . $config['dn']['people'];
            break;
        case "memberof": // cherche si user ou group est membre du groupe branch
            if (! empty($config['etab_ou']) && preg_match("/Equipe_|Profs|Administratifs/", $branch)) {
                $filter = "(&(cn=" . $name . ")(objectclass=user)(memberof=cn=" . $config['etab_ou'] . "," . $config['dn']['etablissements'] . ")(|(memberof=cn=" . $branch . "*)(memberof=" . $branch . ")))";
            } else {
                $filter = "(&(cn=" . $name . ")(objectclass=user)(|(memberof=cn=" . $branch . "*)(memberof=" . $branch . ")))";
            }
            $ldap_attrs = array(
                "cn", // login
                "displayname", // Prenom Nom
                "givenname", // prenm
                "sn", // Nom
                "physicaldeliveryofficename",
                "mail",
                "memberof",
                "pwdlastset",
                "userprincipalname",
                "objectguid"
            );
            if (preg_match("/Equipe_|Profs|Administratifs/", $branch)) {
                $config_local = $config;
                $config = get_config($config, true, true);
                $branch = $config['dn']['people'];
                $config = $config_local;
            } else {
                $branch = $config['dn']['people'];
            }
            break;
        case "pp": // cherche si user ou group est pp de la classe $branch
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=PP_*))";
            else
                $filter = "(&(objectclass=group)(cn=PP_*)(|(cn=PP_" . $name . ")(cn=" . $name . ")(member=cn=" . $name . "*)))";
            $ldap_attrs = array(
                "cn", // login
                "member"
            );
            $branch = $config['dn']['groups'];
            break;

        case "dn":
            $ldap_attrs = array(
                "cn",
                "samaccountname",
                "memberof",
                "member",
                "description"
            );
            $filter = "(cn=*)";
            $branch = $name;
            break;
        case "machine":
            $leases = import_dhcp_leases($config);
        case "machine_fast":
            $ldap_attrs = array(
                "cn", // nom d'origine'displayname' => $etab['name'],
                "samaccountname", // Nom netbios type NOM$
                "dnsHostname", // FDQN
                "location", // Emplacement (prise murale)
                "lastlogon", // derniere connexion
                "description", // Description de la machine
                "iphostnumber", // adresse ip reservée
                "networkaddress", // adresse mac
                "memberof", // appartenance aux groupes
                             // "netbootinitialization", // action IPXE (id)
                "netbootguid", // action programmée (token/uuid pxe)
                "operatingsystem", // Windows
                "operatingsystemversion", // build version
                "objectguid"
            );
            $filter = "(&(objectclass=computer)(|(cn=" . $name . ")(samaccountname=" . $name . ")(samaccountname=" . strtoupper($name) . "$)(dnshostname=" . $name . "." . $config['domain'] . ")(dnshostname=" . $name . ")(iphostnumber=" . $name . ")(networkaddress=" . $name . ")(netbootguid=" . $name . ")(dn=" . $name . ")))";
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "machine_parc":
            $leases = import_dhcp_leases($config);
            $type = "machine";
            $ldap_attrs = array(
                "cn", // nom d'origine
                "samaccountname", // Nom netbios type NOM$
                "dnsHostname", // FDQN
                "location", // Emplacement
                "lastlogon", // derniere connexion
                "description", // Description de la machine
                "iphostnumber", // adresse ip reservée
                "networkaddress", // adresse mac
                "memberof", // appartenance aux groupes
                             // "netbootinitialization", // action IPXE (id)
                "netbootguid", // empreinte bios (UUID bios)
                "operatingsystem", // Windows
                "operatingsystemversion", // build version
                "objectguid"
            );
            $filter = "(&(objectclass=computer)(|(memberof=" . $name . ")))";
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "remote":
            $ldap_attrs = array(
                "cn", // nom d'origine
                "member", // appartenance aux groupes
                "guacConfigProtocol", // protocole
                "guacConfigParameter" // Parametres
            );
            $filter = "(&(objectclass=guacConfigGroup)(|(cn=" . $name . ")(cn=remote_" . $name . ")))";
            $branch = "ou=rdp," . $config['ldap_base_dn'];
            break;

        case "ou":
            $filter = "(&(objectclass=organizationalunit)(cn=" . $name . "))";
            $branch = $config['dn']['people'];
            $ldap_attrs = array(
                "ou",
                "description"
            );
            break;
        case "etablissement":
            $branch = $config['dn']['etablissements'];
            if ($name == "*")
                $filter = "(objectclass=group)";
            else
                $filter = "(&(objectclass=group)(|(cn=" . $name . ")(info=" . $name . ")))"; // (member=cn=" . $name . "*)))";
            $ldap_attrs = array(
                "cn", // UAI
                "info", // Id GPEI
                "displayname", // nom
                "description", // typeEtab nom ville
                "samaccountname",
                "member",
                "memberof", // bassin
                "telephonenumber", // phone
                "textencodedoraddress" // coordonnées géographiques
            );
            break;

        case "salle":
            $filter = "(&(objectclass=organizationalunit)(ou=" . $name . "))";
            if ($branch == "all") {
                $branch = $branch = $config['dn']['computers'];
            }
            $ldap_attrs = array(
                "cn",
                "ou",
                "description",
                "objectguid"
            );
            break;
        case "parc":
            $filter = "(&(objectclass=group)(samaccountname=" . $name . "))";
            $branch = $config['dn']['parcs'];
            $ldap_attrs = array(
                "cn",
                "description",
                "member",
                "samaccountname",
                "objectguid"
            );
            break;
        case "group":
            if ($name == "*")
                $filter = "(objectclass=group)";
            else
                $filter = "(&(objectclass=group)(|(samaccountname=" . $name . ")(samaccountname=*_" . $name . ")(cn=" . $name . ")(cn=*_" . $name . ")))"; // (member=cn=" . $name . "*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "samaccountname",
                "member",
                "memberof"
            );
            // if (preg_match("/Equipe_|Profs|Eleves|Administratifs/", $name)) {
            if (preg_match("/(Profs|Eleves|Administratifs)/", $name)) {
                $config_local = $config;
                $config = get_config($config, true, true);
                $branch = $config['dn']['groups'];
                $config = $config_local;
            }

            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "classe":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=Classe_*))";
            else
                $filter = "(&(objectclass=group)(|(cn=Classe_" . $name . ")(cn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['classes_rdn'] . "," . $config['dn']['groups'];
            break;
        case "equipe":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=Equipe_*))";
            else
                $filter = "(&(objectclass=group)(cn=Equipe_*)(|(cn=Equipe_" . $name . ")(cn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['equipes_rdn'] . "," . $config['dn']['groups'];
            break;
        case "matiere":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=Matiere_*))";
            else
                $filter = "(&(objectclass=group)(cn=Matiere_*)(|(cn=Matiere_" . $name . ")(cn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['matieres_rdn'] . "," . $config['dn']['groups'];
            break;
        case "cours":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=Cours_*))";
            else
                $filter = "(&(objectclass=group)(cn=Matiere_*)(|(cn=Cours_" . $name . ")(cn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['cours_rdn'] . "," . $config['dn']['groups'];
            break;
        case "projet":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=Projet*))";
            else
                $filter = "(&(objectclass=group)(cn=Projet_*)(|(cn=Projet_" . $name . ")(cn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['projets_rdn'] . "," . $config['dn']['groups'];
            break;
        case "autre":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=*))";
            else
                $filter = "(&(objectclass=group)(|(cn=" . $name . ")(cn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['other_groups_rdn'] . "," . $config['dn']['groups'];
            break;
        // obsolete, on n'utilise plus les gpo
        case "printer":
            if ($name == "*")
                $filter = "(objectclass=msPrint-ConnectionPolicy)";
            else
                $filter = "(&(objectclass=msPrint-ConnectionPolicy)(|(printername=" . $name . ")(cn=" . $name . ")))";
            $ldap_attrs = array(
                "cn",
                "printername",
                "servername",
                "printattributes",
                "uncname"
            );
            if ($branch == "all")
                $branch = "CN=Policies,CN=System," . $config['ldap_base_dn'];
            else
                $branch = $config['dn']['people'];

            $branch = $branch . ",CN=Policies,CN=System," . $config['ldap_base_dn'];
            break;
        case "container":
            if ($name == "*")
                $filter = "(objectclass=container)";
            else
                $filter = "(&(objectclass=container)(cn=" . $name . "))";
            $ldap_attrs = array(
                "cn"
            );
            if ($branch == "all")
                $branch = "CN=Policies,CN=System," . $config['ldap_base_dn'];
            else
                $branch = $branch . ",CN=Policies,CN=System," . $config['ldap_base_dn'];
            break;
        case "delegation":
            if ($name == "*")
                $filter = "(objectclass=group)";
            else
                $filter = "(&(objectclass=group)(|(member=" . $name . ")(member=cn=" . $name . "*)))";
            $ldap_attrs = array(
                "cn",
                "samaccountname"
            );
            $branch = $config['dn']['delegations'];
            break;
        case "right":
            if ($name == "*") {
                $filter = "(objectclass=group)";
            } else {
                if (! is_array($name)) {
                    $filter = "(&(objectclass=group)(|(cn=" . $name . ")(member=cn=" . $name . "*)(member=" . $name . ")))";
                } else {
                    $filter = "(&(objectclass=group)(|";
                    foreach ($name as $value) {
                        $filter .= "(cn=" . $value . ")(member=cn=" . $value . "*)(member=" . $value . ")";
                    }
                    $filter .= "))";
                }
            }
            $branch = $config['dn']['rights'];
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            break;
        case "type": // recherche par dn uniquement
            $filter = "(|(objectclass=group)(objectclass=person)(objectclass=computer)(objectclass=organizationalunit))";
            $ldap_attrs = array(
                "objectclass"
            );
            $branch = $name;
            break;
        case "gpo":
            /*
             * liste les gpo, directeent en ldap pour eviter d'avoir à parser les resultats de samba-tool
             */
            if ($name == "*")
                $filter = "(objectclass=grouppolicycontainer)";
            else
                $filter = "(&(objectclass=grouppolicycontainer)(|(cn=" . $name . ")(displayname=" . $name . ")))";
            $ldap_attrs = array(
                "cn",
                "displayname",
                "gpcfilesyspath",
                "versionnumber",
                "gpcuserextensionnames",
                "gpcmachineextensionnames",
                "gpcfunctionalityversion",
                "flags"
            );
            $branch = "CN=Policies,CN=System," . $config['ldap_base_dn'];
            break;
        case "site":
            /*
             * liste les sites directeent en ldap pour eviter d'avoir à parser les resultats de samba-tool
             */
            if ($name == "*")
                $filter = "(objectclass=site)";
            else
                $filter = "(&(objectclass=site)(cn=" . $name . "))";
            $ldap_attrs = array(
                "cn",
                "description"
            );
            $branch = "CN=Sites,CN=Configuration," . $config['ldap_base_dn'];
            break;
        case "subnet":
            /*
             * liste les sous reseaux directeent en ldap pour eviter d'avoir à parser les resultats de samba-tool
             */
            if ($name == "*")
                $filter = "(objectclass=Subnet)";
            else
                $filter = "(&(objectclass=Subnet)(|(cn=" . $name . ")(siteobject=CN=" . $name . ",CN=Sites,CN=Configuration," . $config['ldap_base_dn'] . ")))";
            $ldap_attrs = array(
                "cn", // IP/CIDR
                "description",
                "siteobject",
                "location" // n° de vlan
            );
            $branch = "CN=Subnets,CN=Sites,CN=Configuration," . $config['ldap_base_dn'];
            break;
        case "sitelink":
            /*
             * liste les sous reseaux directeent en ldap pour eviter d'avoir à parser les resultats de samba-tool
             */
            if ($name == "*")
                $filter = "(objectclass=sitelink)";
            else
                $filter = "(&(objectclass=sitelink)(cn=" . $name . "))";
            $ldap_attrs = array(
                "cn",
                "description",
                "cost",
                "replinterval",
                "sitelist"
            );
            $branch = "CN=IP,CN=Inter-Site Transports,CN=Sites,CN=Configuration," . $config['ldap_base_dn'];
            break;

        default:
            $filter = "(cn=" . $name . ")";
            $ldap_attrs = array(
                "cn"
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
    }
    $ldap_attrs = array_merge($ldap_attrs, $attrs);
    $ret = array();
    $error = "";
    // $config['bind'] = bind_ad_gssapi($config, $error);
    if ($config['bind']) {
        if ($scope == "subtree") {
            $result = ldap_search($config['bind'], $branch, $filter, $ldap_attrs);
        } elseif ($scope == "base") {
            $result = ldap_list($config['bind'], $branch, $filter, $ldap_attrs);
        }
        if ($result) {
            $info = ldap_get_entries($config['bind'], $result);
            ldap_free_result($result);
            foreach ($info as $key1 => $entry) {
                if (is_array($entry)) {
                    foreach ($entry as $key2 => $attr) {
                        if ("$key2" == "dn") {
                            $ret[$key1][$key2] = $attr;
                            $ret[$key1]['etab'] = ldap_dn2uai($attr);
                        }
                        if (is_array($attr)) {
                            if ((isset($map[$key2])) && ($type != "gpo")) {
                                $key = $map[$key2];
                            } else {
                                $key = $key2;
                            }
                            switch ($key) {
                                case "physicaldeliveryofficename":
                                    if (isset($attr[0])) {
                                        $tmp = preg_split('/,/', $attr[0], 4);
                                        if (count($tmp) > 0) {
                                            $ret[$key1]['sexe'] = (isset($tmp[1]) ? $tmp[1] : "");
                                            $ret[$key1]['date'] = (isset($tmp[0]) ? $tmp[0] : "");
                                        } else {
                                            $ret[$key1]['hash'] = $attr[0];
                                        }
                                    }
                                    break;
                                case "title":
                                    if (isset($attr[0])) {
                                        $tmp = preg_split('/,/', $attr[0], 2);
                                        if (count($tmp) > 0) {
                                            $ret[$key1]['externalId'] = $tmp[1] ?? "";
                                            $ret[$key1]['id'] = $tmp[0] ?? "";
                                        }
                                    }
                                    break;
                                case "employeenumber":
                                    if (isset($attr[0])) {
                                        $tmp = preg_split('/,/', $attr[0], 4);
                                        if (count($tmp) > 0) {
                                            $ret[$key1]['Id GPEI'] = $tmp[1] ?? "";
                                            $ret[$key1]['Id Siecle'] = $tmp[0] ?? "";
                                            $ret[$key1]['Id ASM'] = $tmp[2] ?? "";
                                            $ret[$key1]['Id Pronote'] = $tmp[3] ?? "";
                                        }
                                    }
                                    break;
                                case "member":
                                case "memberof":
                                case "guacconfigparameter":
                                case 'serviceprincipalname':
                                case 'sitelist':
                                    foreach ($attr as $key3 => $gdn) {
                                        if ("$key3" != 'count') {
                                            $ret[$key1][$key][] = $gdn;
                                        }
                                    }
                                    break;
                                case "objectclass":
                                    foreach ($attr as $key3 => $obj) {
                                        if (("$obj" == "user") || ("$obj" == "computer") || ("$obj" == "organizationalunit") || ("$obj" == "group")) {
                                            $ret[$key1][$key][] = $obj;
                                        }
                                    }
                                    break;
                                default:
                                    if (isset($attr[0])) {
                                        // $ret[$key1][$key] = utf8_decode($attr[0]);
                                        $ret[$key1][$key] = $attr[0];
                                    }
                            }
                        }
                    }
                    if ($type == "machine") {
                        set_machine_attrs($config, $leases, $ret[$key1]);
                    }
                }
            }
            // on ne garde que la correspondance exacte samaccountname si elle existe
            if ($type == "group") {
                foreach ($ret as $g) {
                    if ($g['samaccountname'] == $name) {
                        unset($ret);
                        $ret[0] = $g;
                    }
                }
            }
        } else {
            trigger_error("***" . $branch . " " . $filter . "***");
            $ret = false;
        }
    } else {
        $ret = false;
        die($error);
    }
    // print "search_ad(" . $name . ", $type):";
    // var_dump($ret);
    apcu_store('search_' . $hash, $ret, 120);
    return $ret;
}

/**
 * Modification d'un objet dans l'AD
 *
 * @Description Modifie un objet dans l'AD ou un des ses attributs : add / del / replace
 * @type
 * @Parametres array $config, string $name le nom de l'objet (cn), string $type le type d'objet à chercher : user, computer, group, classe, equipe, projet, parc, rights
 * array $attrs tableau associatif des attributs à changer (format ldapmodify), string $mode type d'opération add / del /replace
 * Pour del aussi, il faut fournir la bonne valeur de l'attribut pour que cela fonctionne. On peut ajouter, modifier, supprimer plusieurs attributs a la fois.
 * @returns  true ou false
 *
 */
function modify_ad(array $config, $name, string $type = "", array $attrs = [], string $mode = "replace")
{
    if (count($attrs) == 0) {
        return true;
    }
    if (! is_array($name)) {
        $res = search_ad($config, $name, $type);
    } else {
        $res = [
            $name
        ];
    }
    // var_dump($res);
    if (count($res) == 1) {
        $dn = $res[0]['dn'];
        // $config['bind'] = bind_ad_gssapi($config);
        if ($config['bind']) {
            switch ($mode) {
                case "add":
                    $result = ldap_mod_add($config['bind'], $dn, $attrs);
                    break;
                case "del":
                    $result = ldap_mod_del($config['bind'], $dn, $attrs);
                    break;
                case "replace":
                    $result = ldap_mod_replace($config['bind'], $dn, $attrs);
                    break;
            }
            // ldap_close($config['bind']);
            if (! $result)
                file_put_contents("/tmp/ldap_err.log", "mode : $mode attrs : " . print_r($attrs, true) . "\n", FILE_APPEND);
            else
                apcu_store("ldap_cache_invalid", true, 60);

            return $result;
        }
    }
    return false;
}

/**
 * supprime un objet dans l'AD
 *
 * @type
 * @Parametres $name - le nom de l'objet (cn ou ou), $type - type d'objet recherché
 * @returns true ou false
 */
function delete_ad($config, $name, $type = "")
{
    if ($type == "dn") {
        $res[0]['dn'] = $name;
    } else {
        if (! is_array($name)) {
            $res = search_ad($config, $name, $type);
        } else {
            $res = [
                $name
            ];
        }
    }
    if ($res) {
        $dn = $res[0]['dn'];

        // $config['bind'] = bind_ad_gssapi($config);
        if ($config['bind']) {
            $ret = ldap_delete($config['bind'], $dn);
            // ldap_close($config['bind']);
            if ($ret) {
                apcu_store("ldap_cache_invalid", true, 60);
            }
            return $ret;
        }
    }
    return false;
}

function clean_group($config, $group)
{
    $res = search_ad($config, $group, "group");
    if (strcasecmp($group . $config['suffix'], $res[0]['samaccountname'])) {
        $attrs = array();
        $attrs['samaccountname'] = $group . $config['suffix'];
        modify_ad($config, $group, "group", $attrs);
    }
}

function fix_spn($config)
{
    $ret = "";
    $res = search_ad($config, "*", "machine_fast", "all", [
        "serviceprincipalname"
    ]);
    foreach ($res as $machine) {
        $doublon = [];
        $nb = [];
        if (is_array(@$machine['serviceprincipalname'])) {
            $machine['serviceprincipalname'] = array_map('strtolower', $machine['serviceprincipalname']);
            foreach ($machine['serviceprincipalname'] as $spn) {

                if (in_array($spn, $machine['serviceprincipalname'])) {
                    @$nb[$spn] ++;
                }
                if (@$nb[$spn] > 1) {
                    $doublon[] = $spn;
                }
            }
            if (count($doublon) > 0) {
                $attrs = [
                    'serviceprincipalname' => $doublon
                ];
                $ret .= $machine['cn'] . " : " . print_r($doublon, true) . "\n";
                modify_ad($config, $machine['cn'], "machine", $attrs, "del");
            }
        }
    }
    return $ret;
}

/**
 * deplace ou renomme un objet dans l'AD
 *
 * @Parametres $name - le nom de l'objet (cn ou ou), $new_dn - nouveau dn complet
 * @type
 * @return true ou false
 */
function move_ad($config, $name, $new_dn, $type = "")
{
    if (! is_array($name)) {
        $res = search_ad($config, $name, $type);
    } else {
        $res = [
            $name
        ];
    }
    if ($res) {
        $dn = $res[0]['dn'];
        $new_dn_elements = preg_split('/,/', $new_dn, 2);
        // $config['bind'] = bind_ad_gssapi($config);
        if ($config['bind']) {
            $ret = ldap_rename($config['bind'], $dn, $new_dn_elements[0], $new_dn_elements[1], true);
            // ldap_close($config['bind']);
            apcu_store("ldap_cache_invalid", true, 60);
            return $ret;
        }
    }
    return false;
}

function trash_user($config, $user)
{
    // $type = type_user($config, $user);
    // if (! empty($type)) {
    // $ou = "ou=" . $type . ",";
    // groupdelmember($config, $user, $type);
    // }
    if ($user['cn'] == "admin")
        return false;
    if (! empty($user['memberof'])) {
        foreach ($user['memberof'] as $group) {
            groupdelmember($config, $user, $group);
        }
    }
    $user = $user['cn'];
    $ret = move_ad($config, $user, "cn=" . $user . "," . $config['dn']['trash'], "user");
    if ($ret) {
        $attrs = array();
        $attrs['useraccountcontrol'] = 514;
        modify_ad($config, $user, "user", $attrs);
    }
    return $ret;
}

function recup_user($config, $user, $cat = "")
{
    $categorie = ($cat == "") ? type_user($config, $user) : $cat;
    $ret = move_ad($config, $user, "cn=" . $user . ",ou=" . $categorie . "," . $config['dn']['people'], "user");
    if ($ret) {
        $attrs = array();
        $attrs['useraccountcontrol'] = 512;
        modify_ad($config, $user, "user", $attrs);
        groupaddmember($config, $user, $categorie);
    }
    return $ret;
}

function trash_users($config)
{
    // $users = search_ad($config, "*", "user", $config['dn']['people']);
    // recherche des comptes orphelins (diff entre les OU et les groupes principaux)
    $cns = array();
    $eleves = search_people_group($config, "Eleves");
    foreach ($eleves as $key => $value) {
        $cns[] = $value["cn"];
    }

    $profs = search_people_group($config, "Profs");
    foreach ($profs as $key => $value) {
        $cns[] = $value["cn"];
    }

    $administratifs = search_people_group($config, "Administratifs");
    foreach ($administratifs as $key => $value) {
        $cns[] = $value["cn"];
    }
    $users = array();
    $people = search_ad($config, "*", "user", $config['dn']['people']);
    foreach ($people as $key => $value) {
        if (! in_array($value["cn"], $cns))
            $users[] = $people[$key];
    }
    $ret = true;
    $users_no_trashables = search_ad($config, "*", "memberof", "no_Trash_user");
    $cns_noTrash = array();
    foreach ($users_no_trashables as $value) {
        $cns_noTrash[] = $value["cn"];
    }
    foreach ($users as $user) {
        $trash = true;
        // if (have_right($config, "no_trash_user", $user['cn'])) {
        // have_right consomme trop de resoources dans la boucle
        $trash = (! in_array($user['cn'], $cns_noTrash));
        // }
        // else {
        // foreach ($user['memberof'] as $groupdn) {
        // if (preg_match("/cn=(Classe_|Equipe_|Cours_|Administratifs|Domain Admins).*/i", $groupdn))
        // $trash = false;
        // }
        // }
        if ($trash) {
            print "trash : " . $user['cn'];
            $ret = trash_user($config, $user);
            if ($ret == 1) {
                echo "<font color='green'> SUCCES</font>" . "<br>\n";
            } else {
                echo "<font color='red'> ECHEC</font>" . "<br>\n";
            }
        }
    }
    return $ret;
}

function check_invalid_passwords($config, &$html)
{
    $users = search_ad($config, "*", "user");
    $ret = true;
    $html = "Verification des mots de passe de " . count($users) . "<br>\n";
    foreach ($users as $user) {
        if (! empty($user['date'])) {
            $attrs = [];
            // on leve la contrainte de temps d'exec (mauvaise idee ?)
            set_time_limit(30);
            if (user_valid_passwd($config, $user['cn'], $user['date']) && $user['pwdlastset'] != 0) {
                $attrs['pwdlastset'] = 0;
                $html .= "Attention " . $user['cn'] . " a sa date de naissance comme mot de passe ! Il devra le changer à la prochaine connexion.<br>\n";
            }
            $attrs['physicaldeliveryofficename'] = password_hash($user['date'], PASSWORD_DEFAULT);
            if (modify_ad($config, $user['cn'], "user", $attrs)) {
                $html .= $user['cn'] . " modifié.<br>";
            } else {
                $ret = false;
                $html .= "Erreur " . $user['cn'] . " non modifié.<br>";
            }
        }
    }
    return $ret;
}

/*
 * retourne le type d'un objet ad
 *
 */
function type_ad($config, $name)
{
    $res = search_ad($config, $name, "type");
    if ($res) {
        if (in_array("user", $res[0]['objectclass']))
            return "user";
        elseif (in_array("group", $res[0]['objectclass']))
            return "group";
        elseif (in_array("organizationalunit", $res[0]['objectclass']))
            return "ou";
    }
    return false;
}

/**
 * Recherche d'utilisateurs dans la branche people
 *
 * @Parametres $filter - Un filtre de recherche permettant l'extraction de l'annuaire des utilisateurs
 * @return Un tableau contenant les utilisateurs repondant au filtre de recherche ($filter)
 *        
 */
function filter_user($config, $filter)
{
    $ret = search_ad($config, $filter, "filter");

    // Tri du tableau par ordre alphabetique
    if (count($ret)) {
        usort($ret, "cmp_nom");
    }
    return $ret;
}

/**
 * Rechercher des groupes selon un critère
 *
 * @description  Recherche une liste de groupes repondants aux criteres fixes par la variable $filter. Les filtres sont les memes que pour ldapsearch.Par exemple (&(cnMember=wawa)(cnMember=toto)) recherche le groupe contenant les utilisateurs wawa et toto.
 * @type
 * @returns Retourne un tableau $groups avec le cn et la description de chaque groupe
 * @Parametres $config, string $filter - Un filtre de recherche permettant l'extraction de l'annuaire des utilisateurs, string $branch="default"
 *
 *
 *
 */
function filter_group($config, $filter, string $branch = "default")
{
    if (preg_match("/(eleves|profs|administratifs)/i", $filter)) {
        $config = get_config($config, true, true);
    }
    if ($branch == "default")
        $branch = $config['dn']['groups'];
    $groups = search_ad($config, "(&(objectclass=group)" . $filter . ")", "filter", $branch);
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de tous les groupes autres
 *
 * @description  Recherche la liste des groupes autres et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_autres($config)
{
    $groups = search_ad($config, "*", "autre");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de tous les groupes projets
 *
 * @description  Recherche la liste des groupes projets et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_projets($config)
{
    $groups = search_ad($config, "*", "projet");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de tous les cours
 *
 * @description  Recherche la liste des cours et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_cours($config)
{
    $groups = search_ad($config, "*", "cours");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de toutes les matières
 *
 * @description  Recherche la liste des groupes matieres et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_matieres($config)
{
    $groups = search_ad($config, "*", "matiere");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de toutes les équipes
 *
 * @description  Recherche la liste des equipes et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_equipes($config)
{
    $groups = search_ad($config, "*", "equipe");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de toutes les classes
 *
 * @description  Recherche la liste des classes et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_classes($config)
{
    $groups = search_ad($config, "*", "classe");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Retourne un tableau avec les attributs d'un groupe
 *
 * @type
 * @Parametres $cn du groupe
 *
 * @returns Un tableau contenant les informations du groupe $cn
 * ['cn'] : nom court
 * ['member']  : membres
 * ['memberof'] : droits de la branche rights
 * ['dn'] : dn complet
 */
function search_group($config, $cn)
{
    if (preg_match("/^cn=/i", $cn)) {
        $ret = search_ad($config, $cn, "dn");
    } else {
        $ret = search_ad($config, $cn, "group", $config['dn']['groups']);
    }
    // var_dump($ret);
    if (count($ret) > 0) {
        return $ret[0];
    } else
        return array();
}

/**
 *
 * @description Retourne un tableau avec les attributs d'un utilisateur (a partir de l'annuaire LDAP)
 * @type
 * @Parametres $cn de l'utilisateur
 * @returns Un tableau contenant les informations sur l'utilisateur (cn)
 *         les groupes sont dans le tableau $res['memberof']
 *
 */
function search_user($config, $cn, $branch = "all")
{
    if (! empty($cn)) {
        $ret = search_ad($config, $cn, "user", $branch);
        if (is_array($ret) && count($ret) > 0 && is_array($ret[0])) {
            return $ret[0];
        } else
            return array();
    } else
        return array();
}

/**
 * donne l'uai de l'établissement, en fonction de son ID (gpei)
 * si id = 0 ou absent renvoie l'uai de l'établissement local (etab_uai) ou 0
 *
 * si id est un uai on le passe direct
 *
 * @param array $config
 *            @ param id
 * @return uai
 */
function etab_uai(array $config, $id = 0)
{
    if ($id == 0) {
        return ($config['etab_ou'] ?? 0);
    }
    if (preg_match("/[0-9]{7}[a-zA-Z]/", $id)) {
        return strtolower($id);
    }
    $etabs = list_etabs($config);
    return $etabs[$id] ?? 0;
}

/**
 * actualise la table id -> uai des établissements
 *
 * Dans le cas d'un établissement unique, on mettra 0
 * Dans le cas d'un serveur de test, on mettra 0000000x
 *
 * En multi-serveurs 0 correspond au serveur central.
 *
 *
 * @param unknown $config
 * @return liste id=> uai
 */
function list_etabs($config)
{
    $etabs = apcu_fetch("etabs");
    if (apcu_fetch("ldap_cache_invalid") || ! is_array($etabs)) {
        $etabs[0] = 0;
        $result = ldap_search($config['bind'], $config['dn']['etablissements'], "(cn=*)", [
            'info',
            'cn'
        ]);
        $info = ldap_get_entries($config['bind'], $result);
        if ($info) {
            ldap_free_result($result);
            $ret = [];
            foreach ($info as $key1 => $entry) {
                if (is_array($entry)) {
                    foreach ($entry as $key => $attr) {
                        if (is_array($attr)) {
                            if (isset($attr[0])) {
                                $ret[$key1][$key] = $attr[0];
                            }
                        }
                    }
                }
            }
            // $se4fs = array_column(search_ad($config, "se4fs-*", "machine_fast"), "etab");
            foreach ($ret as $e) {
                if (isset($e['info'])) {
                    $etabs[$e['info']] = $e['cn'];
                }
            }
            apcu_store("etabs", $etabs);
        }
    }
    return $etabs;
}

/**
 * actualise la table id -> uai des établissements
 *
 * Dans le cas d'un établissement unique, on mettra 0
 * Dans le cas d'un serveur de test, on mettra 0000000x
 *
 * En multi-serveurs 0 correspond au serveur central.
 *
 *
 * @param unknown $config
 *
 * @return liste id=> uai
 */
function etab_to_name($config, $uai = 0)
{
    $etabs = apcu_fetch("etabs_name");
    if (apcu_fetch("ldap_cache_invalid") || ! is_array($etabs)) {
        $etabs[0] = $config['etab_name'] ?? "";
        $result = ldap_search($config['bind'], $config['dn']['etablissements'], "(cn=*)", [
            'description',
            'cn'
        ]);
        $info = ldap_get_entries($config['bind'], $result);
        if ($info) {
            ldap_free_result($result);
            $ret = [];
            foreach ($info as $key1 => $entry) {
                if (is_array($entry)) {
                    foreach ($entry as $key => $attr) {
                        if (is_array($attr)) {
                            if (isset($attr[0])) {
                                $ret[$key1][$key] = $attr[0];
                            }
                        }
                    }
                }
            }
            foreach ($ret as $e) {
                if (isset($e['cn'])) {
                    $etabs[$e['cn']] = $e['description'];
                }
            }
            apcu_store("etabs_name", $etabs);
        }
    }
    $info = $etabs[$uai] ?? "";
    if (! empty($info)) {
        $info = "(" . $info . ")";
    }
    return $info;
}

/**
 * retourne la liste des établissement activés
 *
 * @param array $config
 * @return array $etabs
 */
function activated_etabs(array $config)
{
    $sites = search_ad($config, "*", "site");
    $se4s = search_ad($config, "se4fs*", "machine_fast");
    $etabs = [];
    foreach ($sites as $site) {

        $found = false;
        foreach ($se4s as $k => $s) {
            if ($s['etab'] == $site['etab']) {
                $etabs[] = [
                    'uai' => $site['etab'],
                    'activated' => true
                ];
                unset($se4s[$k]);
                $found = true;
                break;
            }
        }
        if (! $found) {
            $etabs[] = [
                'uai' => $site['etab'],
                'activated' => false
            ];
        }
    }
    return $etabs;
}

/**
 * retourne la liste des établissements d'un utilisateur
 * uniquement parmi les établissements activés
 * l'établissmeent de rattachement est en premier
 *
 * @param array $config
 * @param string $login
 * @return array $e
 */
function my_etabs(array $config, string $login)
{
    $etabs = activated_etabs($config);
    $user = search_user($config, $login);
    $e = [];
    foreach ($etabs as $etab) {
        foreach ($user['memberof'] as $k => $m) {
            if (preg_match("/cn=" . $user['etab'] . ",/i", $m)) {
                array_unshift($e, $etab);
                unset($user['memberof'][$k]);
                break;
            } elseif ($etab['uai'] != 0 && preg_match("/cn=" . $etab['uai'] . ",/i", $m)) {
                $e[] = $etab;
                unset($user['memberof'][$k]);
                break;
            }
        }
    }
    return $e;
}

/**
 * Recherche d'une machine
 *
 * @description Retourne un tableau avec les attributs d'un ordinateur (a partir de l'annuaire LDAP)
 * @type
 * @Parametres cn,dn ou displayname, recherche de l'ip depuis le dns ou le dhcp en option
 * @returns Un tableau contenant les informations sur la machine (cn ou dn ou dsiplayname)
 *         les dn groupes sont dans le tableau $res['memberof']
 *
 */
function search_machine(array $config, string $cn, bool $ip = false, $force = false)
{
    if ($ip) {
        $type = "machine";
    } else {
        $type = "machine_fast";
    }
    $m = search_ad($config, $cn, $type)[0] ?? [];

    if (isset($m['attrs'])) {
        modify_ad($config, $m['cn'], "machine", $m['attrs'], $m['mod']);
        unset($m['attrs']);
        unset($m['mod']);
    }
    return $m;
}

function check_computer($config, $cn)
{
    $res = true;
    if (preg_match("/^(se4fs|se4ad).*/i", $cn)) {
        // surtout on ne fait rien si la machine est un serveur !
        return true;
    }
    $liste = search_ad($config, $cn, "machine");
    $del = $ret = array();
    // il peut y avoir des doublons de cn, par exemple dans le cas d'une intégration manuelle. On les efface (sauf 1 bien sur...)!
    foreach ($liste as $machine) {
        if ((count($liste) > 1) && (strtolower($machine['cn']) == ldap_hostname($config, $machine)) && empty($machine['netbootguid'] && empty($machine['networkaddress'] && empty($machine['iphostnumber'])))) {
            // la machine n'a ni uuid bios, ni ip, ni adresse mac !
            $del[] = $machine;
        } else {
            $ret[] = $machine;
        }
    }
    if (count($ret) > 0) {
        foreach ($del as $d) {
            delete_ad($config, $d['dn'], 'dn');
            dns_delete($config, $d, true);
        }
        $del = [];
        foreach ($ret as $r) {
            if (strcasecmp(ldap_dn2oudn($r['dn']), "cn=computers," . $config['ldap_base_dn']) == 0) {
                $move = $r;
            } else {
                $del[] = $r;
            }
        }
        if (! empty($move)) {
            if (! empty($del)) {
                foreach ($del as $d) {
                    delete_ad($config, $d['dn'], 'dn');
                    dns_delete($config, $d, true);
                }
                $dn = $d['dn'];
            } else {
                $dn = "cn=" . $cn . "," . $config['dn']['computers'];
            }
            $res = move_ad($config, $cn, $dn, "machine");
        }
    } elseif (count($del) > 0 && strcasecmp(ldap_dn2oudn($del[0]['dn']), "cn=computers," . $config['ldap_base_dn']) == 0) {
        $dn = "cn=" . $cn . "," . $config['dn']['computers'];
        $res = move_ad($config, $cn, $dn, "machine");
    }
    if (! empty($ret[0]['iphostnumber'])) {
        $res &= dns_add($config, $ret[0]);
        $res &= dns_add_ptr($config, $ret[0]['cn'], $ret[0]['iphostnumber']);
    }
    return $res;
}

/**
 * Recherche d'une machine
 *
 * @description Retourne un tableau avec les attributs d'un ordinateur (a partir de l'annuaire LDAP)
 * @type
 * @Parametres cn,dn ou displayname, recherche de l'ip depuis le dns ou le dhcp en option
 * @returns Un tableau contenant les informations sur la machine (cn ou dn ou dsiplayname)
 *         les dn groupes sont dans le tableau $res['memberof']
 *
 */
function search_machine_old(array $config, string $cn, bool $ip = false, $force = false)
{
    $store = [];
    $store = apcu_fetch("machines");
    if (! $force && isset($store[$cn]['cn']) && (! $ip || (isset($store[$cn]['state']) && isset($store[$cn]['iphostnumber'])))) {
        return $store[$cn];
    } else {
        if ($ip) {
            $type = "machine";
        } else {
            $type = "machine_fast";
        }
        lock('machines');
        $store = apcu_fetch("machines");
        $liste = search_ad($config, $cn, $type);
        $del = $ret = $m = array();
        // il peut y avoir des doublons de cn, par exemple dans le cas d'une intégration manuelle. On les efface (sauf 1 bien sur...)!
        foreach ($liste as $machine) {
            if ((count($liste) > 1) && (strtolower($machine['cn']) == ldap_hostname($config, $machine)) && empty($machine['netbootguid'])) {
                $del[] = $machine;
            } else {
                $ret[] = $machine;
            }
        }
        if (count($ret) > 0) {
            foreach ($del as $d) {
                delete_ad($config, $d['dn'], 'dn');
                unset($store[$d['cn']]);
                dns_delete($config, $d, true);
            }
            if (count($ret) > 1) {
                foreach ($ret as $key => $r) {
                    if (strcasecmp(ldap_dn2oudn($r['dn']), "cn=computers," . $config['ldap_base_dn']) == 0) {
                        delete_ad($config, $r['dn'], 'dn');
                        unset($store[$r['cn']]);
                        dns_delete($config, $r, true);
                        unset($ret[$key]);
                    }
                }
            }
            $m = $ret[0];
            if (isset($m['attrs'])) {
                modify_ad($config, $m['cn'], "machine", $m['attrs'], $m['mod']);
            }
            // pose problème ? @TODO A vérifier
            // dns_delete($config, $m, false);
        } elseif (count($del) > 0) {
            $m = $del[0];
            array_shift($del);
            foreach ($del as $d) {
                delete_ad($config, $d['dn'], 'dn');
                unset($store[$d['cn']]);
                dns_delete($config, $d, true);
            }
        }
        $store[$cn] = $m;
        apcu_store("machines", $store, 5);
        unlock('machines');
        return $m;
    }
}

function set_machine_attrs($config, array $leases, array &$ret, $ip = true)
{
    putenv('RES_OPTIONS=retrans:1 retry:1 timeout:1 attempts:1');
    $ret['cn'] = strtolower($ret['cn']);
    if (! empty($ret['netbootguid'])) {
        $ret['netbootguid'] = to_guid($ret['netbootguid']);
    }
    if ($ip) {
        if (! empty($ret['iphostnumber']) && ! empty($ret['networkaddress'])) {
            $ret['state'] = "reservation";
        } else {
            $lease = get_dhcp_lease($config, $leases, $ret['cn']);
            if (! $lease && isset($ret['networkaddress']))
                $lease = get_dhcp_lease($config, $leases, $ret['networkaddress']);
            if ($lease) {
                if (! isset($ret['networkaddress'])) {
                    $ret['networkaddress'] = $lease['mac'];
                    $ret['attrs'] = [
                        'networkaddress' => $lease['mac']
                    ];
                    $ret['mod'] = 'add';
                } elseif ($ret['networkaddress'] != $lease['mac']) {
                    $ret['networkaddress'] = $lease['mac'];
                    $ret['attrs'] = [
                        'networkaddress' => $lease['mac']
                    ];
                    $ret['mod'] = 'replace';
                }
                $ret['iphostnumber'] = $lease['ip'];
                $ret['state'] = $lease['state'];
            } elseif (isset($ret['dnshostname'])) {
                if ($ip = gethostbynamel($ret['dnshostname'])) {
                    $ret['iphostnumber'] = $ip[0];
                    $ret['state'] = "dns";
                } else {
                    $ret['state'] = "unknown";
                    $ret['iphostnumber'] = "";
                }
                // pas de dns, on tente le dhcp
            } else {
                $ret['state'] = "unknown";
                $ret['iphostnumber'] = "";
            }
        }
    }

    return true;
}

function register_machine_hardware($config, array $machine, string $uuid)
{
    if (empty($machine['netbootguid'])) {
        // l'enregistrement machine n'est pas complet
        $machine['netbootguid'] = $uuid;
        modify_ad($config, $machine['cn'], "machine", [
            'netbootguid' => from_guid($uuid)
        ], "add");
    }
    return $machine;
    /*
     * } elseif ($uuid != $machine['netbootguid']) {
     * // la machine qui se connecte n'a pas le bon uuid ! ce n'est pas normal !
     * // return false;
     * return $machine;
     * } else {
     * // tout est ok
     * return $machine;
     * }
     */
}

/**
 * Ajout d'une machine
 *
 * @description Ajoute une machine dans l'AD si besoin, sinon renvoie true.
 * @type
 * @Parametres cn,dn ou displayname, recherche de l'ip depuis le dns ou le dhcp en option
 * @returns boolean
 *
 */
function create_machine($config, $name, $ou_rdn = "", $description = "reservation dhcp uniquement")
{
    $name = add_hostname_suffix($config, $name);
    if (count(search_ad($config, $name, "machine")) == 0) {
        // Prépare les données
        if ($ou_rdn == "")
            $ou_rdn = $config['computers_rdn'];
        $info = array();
        $info["cn"] = "$name";
        $info["objectclass"] = array(
            "top",
            "computer"
        );
        // $info["samaccountname"] = $name . $config['suffix'] . "$";
        $info["samaccountname"] = $name . "$";
        // $info["samaccounttype"] = 0x30000001;
        $info["description"] = $description;
        $info["useraccountcontrol"] = 0x1000;

        // Ajout
        // $config['bind'] = bind_ad_gssapi($config);
        $ret = ldap_add($config['bind'], "cn=" . $name . "," . $ou_rdn . "," . $config['ldap_base_dn'], $info);
        // $err = ldap_error($config['bind']);
        // ldap_close($config['bind']);

        // ajout dans les parcs hérités de la salle
        if ($ret) {
            $parcs = explode(',', $ou_rdn);
            array_pop($parcs);
            foreach ($parcs as $parc) {
                groupaddmember($config, $name, explode('=', $parc)[1] . $config['suffix']);
            }
            apcu_store("ldap_cache_invalid", true, 60);
            search_machine($config, $name, true, true);
        }
        return $ret;
    } else
        return true;
}

/**
 * Ajout d'une utilisateur
 *
 * @description joute un utilisateur dans l'AD si besoin, sinon renvoie true.
 * @type
 * @Parametres dn, $info
 *
 * @returns boolean
 *
 */
function create_ad_user($config, $user)
{
    $dn = $user['dn'];
    unset($user['dn']);
    $user['objectclass'] = array(
        "top",
        "user"
        // "person",
        // "organizationalPerson",
    );
    $user['samaccountname'] = $user['cn'];
    $ret = ldap_add($config['bind'], $dn, $user);
    // $err = ldap_error($config['bind']);

    if ($ret) {
        apcu_store("ldap_cache_invalid", true, 60);
    }
    return $ret;
}

/**
 * Verification du couple login / mot de passe d'un utilisateur
 *
 *
 * @Parametres
 * @return * si passwd ok :
 *         pas de newpassword : -> 1
 *         si le mdp doit être changé -> -1
 *         si passwd pas ok -> 0
 *         si passwd ok et newpasswd on change -> 1
 *         si erreur -> 0
 */
function user_valid_passwd($config, $login, $password) // , $newpassword = "")
{
    $config = get_config($config, true, true);
    $url = ad_url($config, "ldaps");
    $ret = 0;
    $user = search_user($config, $login);
    if (count($user) > 0) {
        if ($user['pwdlastset'] == 0) { // || ! empty($newpassword)) {
                                        // mdp à changer
                                        // if (usersetpassword($config, $login, $newpassword, false, $password)) {
                                        // $ret = 1;
                                        // } else {
            $ret = - 1;
            // }
        } else {
            // test simple
            $login_dn = $user['dn'];
            $ds = ldap_connect($url);
            if ($ds) {
                $r = @ldap_bind($ds, $login_dn, $password);
                if ($r) {
                    $ret = 1;
                }
                ldap_close($ds);
            }
        }
    }
    return $ret;
}

function user_valid_passwd_old($config, $login, $password, $newpassword = "")
{
    $mod = false;
    $config = get_config($config, true, true);
    $url = ad_url($config, "ldaps");
    $ret = 0;
    $user = search_user($config, $login);
    if (count($user) > 0) {
        if ($user['pwdlastset'] == 0) {
            $attr['pwdlastset'] = - 1;
            if (modify_ad($config, $user['cn'], "user", $attr, "replace")) {
                $mod = true;
            }
        }
        $login_dn = $user['dn'];
        $ds = ldap_connect($url);
        if ($ds) {
            $r = @ldap_bind($ds, $login_dn, $password);
            if ($r) {
                if (! empty($newpassword)) {
                    if (usersetpassword($config, $login, $newpassword)) {
                        $mod = false;
                        $ret = 1;
                    } else {
                        $ret = - 1;
                    }
                } else {
                    $ret = 1;
                }
            } else {
                $ret = 0;
            }
            ldap_close($ds);
        } else {
            $ret = 0;
        }
        if ($mod) {
            $attr['pwdlastset'] = 0;
            if (modify_ad($config, $user['cn'], "user", $attr)) {
                $ret = 1;
            }
        }
    }
    return $ret;
}

/**
 * Recherche catégorie utilisateur
 *
 * @description retourne le groupe principal d'un utilisateur
 * @type
 * @Parametres array $config, string|array $user
 * @returns string
 */
function type_user(array $config, $user)
{
    $match = [];
    if (is_array($user)) {
        if (preg_match("/ou=(Eleves|Profs|Administratifs).*/i", $user['dn'], $match)) {
            return $match[1];
        }
    } else {
        $res = search_user($config, $user);
        if (count($res) > 0) {
            $match = array();
            $groupsdn = (isset($res['memberof'])) ? $res['memberof'] : array();
            foreach ($groupsdn as $groupdn) {
                if (preg_match("/cn=(Eleves|Profs|Administratifs).*/i", $groupdn, $match)) {
                    return $match[1];
                }
            }
            if (preg_match("/ou=(Eleves|Profs|Administratifs).*/i", $res['dn'], $match)) {
                return $match[1];
            }
        }
    }
    return "";
}

/**
 *
 * Vérification droit ldap pour utilisateur
 *
 * @description Recherche si l'utilisateur courant a au moins un des droits type1|type2..
 * vrai dans tous les cas si l'utilisateur a le droit se3_is_admin
 * @type
 * @Parametres right
 * @returns true ou false
 */
function have_right($config, $type, $user = "login")
{
    if ($user == "login")
        $user = $config['login'] ?? $config['login'];
    $rights = list_rights($config, $user);
    $typearr = explode("|", "$type");
    $typearr[] = "se3_is_admin";
    foreach ($typearr as $right) {
        if (in_array($right, $rights)) {
            return true;
        }
    }
    return false;
}

/**
 * Lister les droits d'un utilisateur
 *
 * @description retourne tous les droits dont dispose $name (user ou groupe)
 * recherche recursive pour les groupes d'appartenance
 * @type
 * @Parametres : $config, $name
 * @Parametres : name - utilisateur ou groupe
 * @returns array des droits
 */
function list_rights($config, $name)
{
    if (empty($name)) {
        return [];
    }
    if (apcu_fetch("ldap_rights_cache_invalid")) {
        apcu_delete(new APCUIterator('#^rights_#'));
        apcu_delete("ldap_rights_cache_invalid");
    }
    if (! $rights = apcu_fetch("rights_" . $name)) {
        $config = get_config($config, true, true);
        $rights = array();
        if ("$name" == "all") {
            $res = search_ad($config, "*", "right");
            foreach ($res as $right) {
                $rights[] = $right['cn'];
            }
            return $rights;
        }
        $user = search_user($config, $name);
        if (count($user) == 0)
            $user = search_group($config, $name);
        $ret = search_ad($config, $name, "right");
        $groups = (isset($user['memberof'])) ? $user['memberof'] : array();
        foreach ($groups as $groupdn) {
            $ret1 = search_ad($config, $groupdn, "right");
            $ret = array_merge($ret1, $ret);
        }
        if ($ret) {
            foreach ($ret as $group) {
                $right = $group['cn'];
                if (! in_array($right, $rights)) {
                    $rights[] = $right;
                }
            }
        }
        apcu_store("rights_" . $name, $rights);
    }
    // if ($inverse) {
    // $not_rights = array();
    // foreach (list_rights($config, "all") as $r) {
    // if (! in_array($r, $rights))
    // $not_rights[] = $r;
    // }
    // return $not_rights;
    // }
    // }
    if (count($rights) == 0) {
        return array();
    } else {
        return $rights;
    }
    // // si user n'est membre d'aucun groupe Rights
    // if ($inverse) {
    // return list_rights($config, "all");
    // } else {
    // return array();
    // }
}

/**
 * Lister les membres d'un groupe
 *
 * @description retourne tous les membres d'un groupe donné
 * @type
 * @Parametres : config, $group
 * @returns array des membres
 */
function list_members_group($config, string $group)
{
    $res = search_people_group($config, $group);
    return ($res);
}

/**
 * Lister les membres disposant d'un droit
 *
 * @description retourne tous les membres disposant d'un droit donné
 * recherche recursive pour les groupes d'appartenance
 * @type
 * @Parametres : config, $right
 * @returns array des dn membres
 */
function list_members_rights($config, $right)
{
    // $res = grouplistmembers($config, $right);
    $groupe = search_ad($config, $right, "right");
    $dns = (isset($groupe[0]['member'])) ? $groupe[0]['member'] : array();
    return $dns;
}

/**
 * Délégue un droit ldap
 *
 * @description donne le droit à l'utilisateur ou a un groupe
 * @type
 * si il l'a dejà par héritage d'un groupe on ne fait rien
 * @Parametres array $config,
 * string|array $cn ou $dn dans le cas d'un groupe
 * string $right
 * @returns true ou false
 */
function add_right($config, $cn, $right)
{
    return groupaddmember($config, $cn, $right);
}

/**
 * Supprime un droit à l'utilisateur
 *
 * @description non récursif : si le droit est hérité il ne sera pas retiré.
 * @type
 * @parametres array $config, string $name, string $right
 * @returns true ou false
 */
function remove_right($config, $cn, $right)
{
    return groupdelmember($config, $cn, $right);
}

/**
 * Fixe la durée du droit admin local
 *
 * @param unknown $config
 * @param unknown $user
 * @param number $duration
 *            2H par défaut, -1 = permanent
 */
function set_local_admin_right(&$config, $user, $duration = 7200)
{
    if ($user == "admin") {
        return - 1;
    }
    if ($duration != - 1) {
        $duration = time() + $duration;
    }
    groupaddmember($config, $user, "domain admins");
    $user = strtr($user, ".", "_");
    $duration = set_param($config, "local_admin_" . $user, $duration, "rights");
    return $duration;
}

function get_local_admin_right($config, $user)
{
    $userm = strtr($user, ".", "_");
    if ($user == "admin") {
        return - 1;
    } elseif (! empty($config['local_admin_' . $userm])) {
        if ($config['local_admin_' . $userm] == - 1) {
            return - 1;
        } elseif ($config['local_admin_' . $userm] > time()) {
            return $config['local_admin_' . $userm] - time();
        } else {
            groupdelmember($config, $user, "domain admins");
            set_param($config, "local_admin_" . $userm, "", "rights");
            return 0;
        }
    } else {
        return 0;
    }
}

// Parcs
// Un parc est composé d'un groupe (pour wpkg et les délégations), et éventuellement d'une OU pour les GPO
// Les OU peuvent être hierarchisées, dans ce cas les groupes parents contiennent aussi les machines des OU enfants

/**
 * recherche les attributs d'un parc (OU + groupe) et les combine
 *
 * l'identifiant unique est le dn, ou le samaccountname. La recherche se fait avec le samaccountname exclusivement. Dans
 * le cas d'une salle on fait l'hypotèse que le groupe associé existe.
 *
 * cas particulier, les branches ou=uai,ou=computers ne sont pas des parcs de type salle, pas de groupe associés dans ou=parcs.
 * Ils sont donc traités indépendamment.
 *
 * Dans le cas mono établissement name=samaccountname.
 *
 * @description indique s'il s'agit d'un parc ou d'une salle
 * @type
 * @Parametres array $config, string $search samaccountname (* pour tous)
 * @returns array salle ou parc
 */
function search_parcs(array $config, string $search, string $type = "all")
{
    if (preg_match("/^(.*)-([0-9]{4}[a-z])/i", $search, $m)) {
        $name = $m[1];
        $suffix = $m[2];
    } else {
        $name = $search;
        $suffix = "";
    }

    $parcs = $salles = array();
    if (($type == "all") || ($type == "salle"))
        $salles = search_ad($config, $name, "salle");
    if (! is_array($salles)) {
        return false;
    }
    $parcs = search_ad($config, $search, "parc");
    if (! is_array($parcs)) {
        return false;
    }
    if (count($salles) + count($parcs) == 0)
        return array();
    else {
        $res = array();
        $i = - 1;
        if (count($salles) > 0) {
            foreach ($salles as $salle) {
                $found = false;
                if (count($res) > 0) {
                    foreach ($res as $re) {
                        if (strtolower($re['name']) == strtolower($salle['ou'])) {
                            $found = true;
                            break;
                        }
                    }
                }

                if (! $found && preg_match("/" . $suffix . "/", $salle['etab'])) {
                    $i ++;
                    $res[$i]['name'] = $salle['ou'];
                    $res[$i]['etab'] = $salle['etab'];
                    $res[$i]['parent'] = ldap_dn2parent($salle['dn']);
                    $res[$i]['parentdn'] = ldap_dn2oudn($salle['dn']);
                    $res[$i]['type'] = "salle";
                    $res[$i]['dn'] = $salle['dn'];
                    $res[$i]['uuid'] = to_guid($salle['objectguid']);
                    if (isset($salle['description']))
                        $res[$i]['description'] = $salle['description'];
                    else
                        $res[$i]['description'] = "Salle " . $res[$i]['name'];
                }
                if (preg_match("/^([0-9]{3})([0-9]{4}[a-z])/i", $salle['ou'], $m) || strcasecmp($salle['ou'], "Computers") == 0) {
                    // cas particulier des branches uai
                    $res[$i]['samaccountname'] = $salle['ou'];
                }
            }
        }
        if (count($parcs) > 0) {
            foreach ($parcs as $k => $parc) {
                /*
                 * if (! empty($config['suffix']) && $parc['cn'] . $config['suffix'] != $parc['samaccountname']) {
                 * $attrs = array();
                 * $attrs['samaccountname'] = $parc['cn'] . $config['suffix'];
                 * modify_ad($config, $parc['cn'], "parc", $attrs);
                 * }
                 */
                $found = false;
                if (count($res) > 0) {
                    foreach ($res as $key => $re) {
                        if (strtolower($re['name']) == strtolower($parc['cn']) && $re['etab'] == $parc['etab']) {
                            $found = true;
                            break;
                        }
                    }
                }
                if (! $found && $type != "salle") {
                    $i ++;
                    $res[$i]['name'] = $parc['cn'];
                    $res[$i]['etab'] = $parc['etab'];
                    $res[$i]['type'] = "parc";
                    $res[$i]['description'] = $parc['description'] ?? "";
                    $res[$i]['uuid'] = to_guid($parc['objectguid']);
                    $res[$i]['dn'] = $parc['dn'];
                    $res[$i]['gdn'] = $parc['dn'];
                    $res[$i]['samaccountname'] = $parc['samaccountname'];
                } elseif (isset($key)) {
                    $res[$key]['description'] = $res[$key]['description'] ?? $parc['description'] ?? "";
                    $res[$key]['gdn'] = $parc['dn'];
                    $res[$key]['samaccountname'] = $parc['samaccountname'];
                }
            }
        }
        // réparation structure AD si besoin.
        foreach ($res as $k => $p) {
            if ($p['type'] == "salle" && preg_match("/^cn=(.*),(.*)$/i", $p['dn'], $m)) {
                trigger_error("Attention salle mal nommee " . $p['dn'] . ", vous devez l'effacer");
                // rename_salle($config, $p, "OU=" . $p['name'] . "," . $p['parent']);
            }
            if (! isset($p['samaccountname'])) {
                // trigger_error("pas de parc associe pour " . $p['type'] . " " . $p["name"] . ". On le cree...");
                // $gsuffix = etab_suffix($p['etab']);
                // groupadd($config, $p["name"], $config['parcs_rdn'], $p['description'], $gsuffix);
            }
        }
        sort_by_key("name", $res);
        return $res;
    }
}

function type_parc(array $config, string $parc)
{
    $parcs = search_parcs($config, $parc);
    return $parcs[0]['type'];
}

/**
 * recherche les parcs pour une machine
 * une recherche "salle" ne renvoie que la salle contenant la machine
 * une recherche "parc" renvoie tous les parcs
 *
 * @type
 * @parametres : machine a rechercher"
 * @parametres : type - "salle", "all" (defaut)"
 * @returns : array(array parcs))
 */
function list_parcs($config, $machine, $type = "all")
{
    $parcs = array();
    if ($machine != "*") {
        if (count($info = search_machine($config, $machine, false)) > 0) {
            $salle = ldap_dn2parent($info['dn']);
            $parcs = search_parcs($config, $salle);
            if ($type == "salle")
                return $parcs;
            if (is_array($info['memberof'])) {
                foreach ($info['memberof'] as $parcdn) {
                    if (preg_match("/" . $config['dn']['parcs'] . "/i", $parcdn)) {
                        $p = search_parcs($config, ldap_dn2sam($parcdn));
                        if ($parcs != $p) {
                            if (! preg_match("/" . $p[0]['gdn'] . "/", $info['dn'])) {
                                $p = search_parcs($config, ldap_dn2sam($parcdn), "parc");
                            }
                            $parcs[] = $p[0];
                        }
                    }
                }
            }
        }
    }
    // sort_by_key("name", $parcs);
    return $parcs;
}

/**
 * recherche les parcs pour une machine
 * une recherche "salle" ne renvoie que nom de l'OU contenant la machine
 * une recherche "parc" renvoie tous les parcs
 *
 * Ne pas utiliser pour une recherche globale, le nom retourné peut ne pas être unique.
 *
 * @type
 * @parametres : machine a rechercher"
 * @parametres : type - "salle", "all" (defaut)"
 * @returns : array(name) nom des parc. Attention il s'agit du nom court, non unique dans le cas multi-etablissmement
 */
function list_parcs_name($config, $machine, $type = "all")
{
    $parcs = array();
    if ($machine != "*") {
        if (count($info = search_machine($config, $machine, false)) > 0) {
            $salle = ldap_dn2cn(ldap_dn2oudn($info['dn']));
            if ($type == "salle")
                return $salle;

            foreach ($info['memberof'] as $parc) {
                if (preg_match("/" . $config['dn']['parcs'] . "/i", $parc)) {
                    $parcs[] = ldap_dn2cn_case($parc);
                }
            }
        }
    }
    // sort_by_key("name", $parcs);
    return $parcs;
}

/**
 * liste les parcs pour toutes les machines
 * renvoie le nom pas unique (pas le samaccountname)
 * utilisée pour la sync avec la base wpkg
 *
 * @returns : array(["parc"] => array, ["objectguid"] => string)
 */
function list_machines_parcs($config)
{
    $machines = search_ad($config, "*", "machine_fast", $config['dn']['computers']);
    if (is_array($machines)) {
        $ret = [];
        foreach ($machines as $machine) {
            $parc = [];
            if (isset($machine['memberof'])) {
                foreach ($machine['memberof'] as $parc_dn) {
                    if (preg_match("/" . $config['dn']['parcs'] . "/i", $parc_dn)) {
                        $parc[] = ldap_dn2cn_case($parc_dn);
                    }
                }
            }
            $dn = ldap_dn2oudn($machine['dn']);
            while (strcasecmp($dn, $config['dn']['computers']) != 0) {
                $ou = ldap_dn2cn(ldap_dn2ou($dn));
                if (! in_array($ou, $parc)) {
                    $parc[] = $ou;
                }
                $dn = ldap_dn2oudn($dn);
            }
            $ret[$machine['cn']]['parc'] = $parc;
            $ret[$machine['cn']]['objectguid'] = to_guid($machine['objectguid']);
        }
    } else {
        $ret = false;
    }
    return $ret;
}

/**
 * liste les salles enfant, et les range par niveau de profondeur
 *
 * la salle d'origine est $ret[0][0]
 * salle : resultat de search_parcs()[0]
 *
 * @returns : [level][[enfant]]
 */
function list_salle_childrens($config, array $salle)
{
    $childs = search_ad($config, "*", "salle", $salle['dn']);
    if (is_array($childs)) {
        $ret = [];
        foreach ($childs as $child) {
            $c = preg_replace("/,?" . $salle['dn'] . "/", "", $child['dn']);
            if ($c) {
                $l = count(explode(',', $c));
            } else {
                $l = 0;
            }
            $ret[$l][] = $child;
        }
    } else {
        $ret = false;
    }
    return $ret;
}

/**
 * creation de parc : crée un groupe, et éventuellement une ou si le type est "salle"
 * le type "salle" permet d'appliquer des GPO, le type "parc" sert pour les delegations et wpkg.
 * Les salles peuvent être imbriquées ( parentou ), afin de gérer des GPO de façon globale.
 * Dans tous les cas un groupe du même nom est créé
 *
 * @parametres
 * @type
 * @returns
 */
function create_parc(array $config, string $parc, string $description = "", string $type = "salle", string $parentou = "")
{
    $res = groupadd($config, $parc, $config['parcs_rdn'], $description, $config['suffix']);
    if ($type == "salle") {
        if (($parentou == "") || (strtolower($parentou) == "computers") || (strtolower($parentou) == @$config['etab_ou'])) {
            $dn = "OU=" . $parc . "," . $config['dn']['computers'];
        } else {
            $parent = search_parcs($config, $parentou, "salle");
            if (isset($parent[0]['dn'])) {
                $dn = "OU=" . $parc . "," . $parent[0]['dn'];
            } else {
                return false;
            }
        }
        $res &= ouadd($config, $dn);
    }
    return $res;
}

/**
 * renomme/déplace une ou=salle
 * cela implique de vider toute l'arborescence vers une arborescent temporaire, puis de la reremplir
 * on ne gère pas les groupes parcs à ce niveau
 *
 * @param unknown $config
 * @param array $salle
 *            issue de search_parcs()[0]
 * @param string $new_dn
 * @return boolean|unknown
 */
function rename_salle($config, array $salle, string $new_dn = "")
{
    $res = true;
    $childs = list_salle_childrens($config, $salle);
    // echo "<pre>" . print_r($childs, true) . "</pre>";
    for ($l = count($childs) - 1; $l >= 0; $l --) {
        foreach ($childs[$l] as $k => $c) {
            $childs[$l][$k]['tmpdn'] = "OU=" . $c['ou'] . "." . $l . "," . $salle['parentdn'];
            $res &= ouadd($config, $childs[$l][$k]['tmpdn']);
            if ($res) {
                echo "création temporaire  de " . $childs[$l][$k]['tmpdn'] . "<br>";
            }
            $machines = list_members_parc($config, $c['ou'], true);
            foreach ($machines as $machine) {
                $res &= move_ad($config, $machine['dn'], "cn=" . $machine['cn'] . "," . $childs[$l][$k]['tmpdn'], "dn");
                if ($res) {
                    echo "déplacement temporaire  de " . $machine['cn'] . "<br>";
                }
            }
            if ($l == 0) {
                if (empty($new_dn)) {
                    // suppression
                    $res &= delete_ad($config, $c);
                    $new_dn = $salle['parentdn'];
                    if ($res) {
                        echo "suppression  de " . $c['ou'] . "<br>";
                    }
                } else {
                    // renommage
                    $res &= move_ad($config, $c, $new_dn);
                    if ($res) {
                        echo "renommage  de " . $c['ou'] . "en $new_dn<br>";
                    } else {
                        echo "renommage  de " . $c['ou'] . "en $new_dn impossible<br>";
                    }
                }
            } else {
                $res &= move_ad($config, $c, "OU=" . $c['ou'] . "," . $salle['parentdn']);
                if ($res) {
                    echo "déplacement  de " . $c['ou'] . " dans " . $salle['parentdn'] . "<br>";
                }
            }
        }
    }
    for ($l = 0; $l < count($childs); $l ++) {
        foreach ($childs[$l] as $k => $c) {
            $dn = preg_replace("/" . $salle['dn'] . "/", $new_dn, $c['dn']);
            if ($l != 0) {
                $res &= move_ad($config, $c, $dn);
                if ($res) {
                    echo "déplacement  de " . $c['ou'] . " dans " . $dn . "<br>";
                }
            }
            $machines = list_members_parc($config, $c['ou'] . "." . $l, true);
            foreach ($machines as $machine) {
                $res &= move_ad($config, $machine['dn'], "cn=" . $machine['cn'] . "," . $dn, "dn");
                if ($res) {
                    echo "remise en place de " . $machine['cn'] . " dans $dn<br>";
                }
            }
            $res &= delete_ad($config, $childs[$l][$k]['tmpdn'], "dn");
            if ($res) {
                echo "effacement de " . $childs[$l][$k]['tmpdn'] . "<br>";
            }
        }
    }
    return $res;
}

/**
 * suppression de parc
 *
 * @description Si il s'agit d'une salle il faut commencer par deplacer les machines vers l'ou parent, Normalement
 * @parametres
 * @type
 * @returns
 *
 *
 */
function delete_parc($config, $parc)
{
    $ret = true;
    $info = search_parcs($config, $parc)[0];
    if ($info['type'] == "salle") {
        $ret = rename_salle($config, $info);
    }
    $ret &= delete_ad($config, $info['gdn'], "dn");
    if ($ret) {
        $res = search_delegations($config, $parc);
        if ($res) {
            foreach ($res as $delegation) {
                $ret &= groupdel($config, $delegation['cn']);
            }
        }
    }
    return $ret;
}

/**
 *
 * @Description deplace une salle ou un parc
 * Cela implique de sortir les machines
 * de tous les parcs anciens parents, et de les remettre dans tous les nouveaux parents
 *
 * @param array $config
 * @param string $newparent
 *            pseudo-samaccountname de la salle parente
 * @param string $parc
 *            samaccountname de la salle
 * @return true
 *
 */
function move_parc(array $config, string $sam, string $parent)
{
    $old_parcs = [];
    $parc = search_parcs($config, $sam, "salle")[0];
    $p = $parc;
    while (strcasecmp($parc['parent'], ldap_dn2cn($config['computers_rdn'])) != 0) {
        $old_parcs[] = $parc['samaccountname'];
        $parc = search_parcs($config, $parc['parent'], "salle")[0];
    }
    $new_parcs = [];
    $parc = search_parcs($config, $parent, "salle")[0];
    $new_parc = $parc;
    while (strcasecmp($parc['samaccountname'], ldap_dn2cn($config['computers_rdn'])) != 0) {
        $new_parcs[] = $parc['samaccountname'];
        $parc = search_parcs($config, $parc['parent'], "salle")[0];
    }
    $remove = array_diff($old_parcs, $new_parcs);
    $add = array_diff($new_parcs, $old_parcs);
    $machines = list_members_parc($config, $p['samaccountname'], false, true);
    // on déplace le OU, on ne touche pas au parc
    $res = rename_salle($config, $p, "OU=" . $p['name'] . "," . $new_parc['dn']);
    if ($res) {
        foreach ($machines as $machine) {
            foreach ($remove as $r) {
                $res &= groupdelmember($config, $machine, $r);
            }
            foreach ($add as $a) {
                $res &= groupaddmember($config, $machine, $a);
            }
        }
    } else {
        echo "Erreur deplacemement : <pre>" . print_r($p, true) . print_r("OU=" . $p['name'] . "," . $new_parc['dn'], true) . "</pre>";
    }
    return $res;
}

/**
 *
 * @Description renomme un parc, sans le déplacer
 * Cela implique de :
 *  - récuperer les applis wpkg si besoin,
 *  - renommer le cn et le ou si salle
 *  - synchroniser la base wpkg
 *  - reinjecter les applis wpkg
 *
 * @param array $config
 * @param string $newparc
 * @param string $member
 * @return true
 *
 */
function rename_parc(array $config, string $sam, string $new_name)
{
    $res = true;
    $parc = search_parcs($config, $sam)[0];
    if ($parc['type'] == "salle") {
        $res &= rename_salle($config, $parc, "OU=" . $new_name . "," . $parc['parentdn']);
    }
    $attrs = [
        'samaccountname' => $new_name . $config['suffix']
    ];
    $res &= modify_ad($config, $sam, "parc", $attrs);
    $res &= move_ad($config, $parc['gdn'], "CN=" . $new_name . "," . $config['dn']['parcs'], "dn");
    return $res;
}

/**
 * liste le contenu d'un parc
 *
 * @description
 * @parametres : parc à lister
 * @parametre : attrs - booleen enregistrements complets ou liste
 * @returns : liste des machines, enregistrements complets des machines, false
 */
function list_members_parc($config, $parc, $attrs = false, $recurse = false, $type = "all")
{
    if ($recurse) {
        $scope = "subtree";
    } else {
        $scope = "base";
    }
    $ret = array();
    $etab = $config['etab_ou'] ?? "";
    if ($type == "all" || $type == "salle") {
        $info = search_parcs($config, $parc);
        if (count($info) > 0) {
            $etab = $info[0]['etab'];
            if ($info[0]['type'] == "salle") {
                $ret = search_ad($config, "*", "machine", $info[0]['dn'], array(), $scope);
                if (! $attrs)
                    $ret = array_column($ret, "cn");
            }

            /*
             * if ($attrs) {
             * $ret = $res;
             * } else {
             * foreach ($res as $machine) {
             * $ret[] = $machine['cn'];
             * }
             * }
             */
        }
    }
    // elseif ($info[0]['type'] = "parc") {

    $machines = search_ad($config, "*", "machine");
    $res = search_ad($config, $parc, "parc");
    if (isset($res[0]['member']) && $res[0]['etab'] == $etab) {
        if ($attrs) {
            foreach ($ret as $ms) {
                foreach ($res[0]['member'] as $key => $m) {
                    if ($m == $ms['dn']) {
                        unset($res[0]['member'][$key]);
                        break;
                    }
                }
            }
            foreach ($machines as $machine) {

                foreach ($res[0]['member'] as $m) {
                    if ($machine['dn'] == $m && $machine['etab'] == $etab) {
                        $ret[] = $machine;
                        break;
                    }
                }
            }
        } else {
            foreach ($res[0]['member'] as $machine) {
                $cn = ldap_dn2cn($machine);

                $ret[] = $cn;
                $ret = array_unique($ret);
            }
        }
    }

    if ($attrs) {
        // $ret = array_unique(array_merge($ret, search_ad($config, $parc['dn'], "machine")));
        sort_by_key("cn", $ret);
    } else {
        // $ret = array_unique(array_merge($ret, array_column(search_ad($config, $parc['dn'], "machine"), "cn")));
        sort($ret);
    }
    return $ret;
}

/**
 * liste le contenu d'un parc
 *
 * @description
 * @parametres : parc à lister
 * @parametre : attrs - booleen enregistrements complets ou liste
 * @returns : liste des machines, enregistrements complets des machines, false
 */
function list_members_parc_fast($config, $parc, $attrs = false)
{
    $ret = array();
    if ($attrs) {
        $ret = search_ad($config, $parc, "machine_parc");
        sort_by_key("cn", $ret);
    } else {
        $ret = array_column(search_ad($config, $parc, "machine_parc"), "cn");
        sort($ret);
    }
    return $ret;
}

/**
 *
 * @Description deplace un membre d'un  parc
 * pour les salles il faut déplacer les machines depuis leur ou d'origine. Cela implique de les sortir aussi du groupe associé.
 * Pour les parcs on ajoute juste au groupe.
 *
 * @param array $config
 * @param string $newparc
 * @param string $member
 * @return true
 */
function move_member_parc(array $config, string $newparc, string $member)
{
    $salles = list_parcs($config, $member, "salle");
    if (isset($salles[0])) {
        $salle = $salles[0];
    } else {
        $salle = false;
    }
    $machine = search_machine($config, $member);
    $newparcs = search_parcs($config, $newparc);
    // echo "salle d'origine " . $salle['samaccountname'] . "<br>";
    if ($newparcs[0]['type'] == "salle" || strcasecmp($newparc, ldap_dn2cn($config['computers_rdn'])) == 0) {
        $newsalle = $newparcs[0];
        // on retire la machine de tous les anciens parcs salles
        while (isset($salle['name']) && strcasecmp($salle['name'], ldap_dn2cn($config['computers_rdn'])) != 0) {
            if (! empty($salle['samaccountname'])) {
                // echo "on enleve de " . $salle['samaccountname'] . "<br>";
                groupdelmember($config, $machine, $salle['samaccountname']);
            }
            $parentparc = search_parcs($config, $salle['parent']);
            $salle = $parentparc[0];
        }
        // on ajoute la machine à tous les parcs salles parents
        while (isset($newsalle['name']) && strcasecmp($newsalle['name'], ldap_dn2cn($config['computers_rdn'])) != 0) {
            // echo "on ajoute a " . $newsalle['samaccountname'] . "<br>";
            groupaddmember($config, $machine, $newsalle['samaccountname']);
            $newparentparc = search_parcs($config, $newsalle['parent']);
            $newsalle = $newparentparc[0];
        }
        // echo "on deplace vers cn=" . $member . "," . $newparcs[0]['dn'] . "<br>";
        return move_ad($config, $member, "cn=" . $member . "," . $newparcs[0]['dn'], "machine");
    } else
        // echo "on ajoute à " . $newparcs[0]['samaccountname'] . "<br>";
        return groupaddmember($config, $machine, $newparcs[0]['samaccountname']);
}

/**
 *
 * @Description ajoute un membre au parc
 * pour les salles il faut déplacer les machines depuis leur ou d'origine. Cela implique de les sortir aussi du groupe associé.
 * Pour les parcs on ajoute juste au groupe.
 * @type
 * @returns true ou false
 */
function add_member_parc($config, $parc, $member)
{
    $parcs = search_parcs($config, $parc);
    $machine = search_machine($config, $member);

    return groupaddmember($config, $machine, $parcs[0]['samaccountname']);
}

/**
 *
 * @Description enlève un membre du parc
 * @parametres
 * @type
 * @returns true ou false
 */
function remove_member_parc($config, $parc, $member)
{
    $parcs = search_parcs($config, $parc);
    $machine = search_machine($config, $member);
    if (! empty($machine)) {
        if ($parcs[0]['type'] == "salle" && strcasecmp(ldap_dn2oudn($machine['dn']), $parcs[0]['dn']) == 0) {
            return move_member_parc($config, $parcs[0]['parent'], $member);
        } else {
            return groupdelmember($config, $machine, $parcs[0]['samaccountname']);
        }
    } else {
        return true;
    }
}

/**
 *
 * @Description teste l'appartenance au parc
 * @parametres $config, $parc, $member
 * @type
 * @returns true ou false
 */
function is_member_parc($config, $parc, $member)
{
    $res = list_parcs($config, $member, "all");
    if ($res) {
        foreach ($res as $value) {
            if (strtolower($value['name']) == strtolower($parc))
                return true;
        }
        return false;
    }
}

function is_dual_boot($config, $machine)
{
    $parcs = $machine['memberof'] ?? [];
    $parcs = array_map('strtolower', $parcs);
    if (in_array(strtolower("CN=windows," . $config['dn']['parcs']), $parcs) && in_array(strtolower("CN=linux," . $config['dn']['parcs']), $parcs)) {
        return true;
    } else {
        return false;
    }
}

function get_os($config, $machine)
{
    $os = [];
    $parcs = $machine['memberof'] ?? [];
    $parcs = array_map('strtolower', $parcs);
    if (in_array(strtolower("CN=windows," . $config['dn']['parcs']), $parcs)) {
        $os[] = "windows";
    }
    if (in_array(strtolower("CN=linux," . $config['dn']['parcs']), $parcs)) {
        $os[] = "linux";
    }
    return $os;
}

function add_dual_boot($config, $name)
{
    set_os($config, $name, "windows");
    set_os($config, $name, "linux");
}

function remove_dual_boot($config, $name)
{
    groupdelmember($config, $name . "$", "windows" . $config['suffix']);
    groupdelmember($config, $name . "$", "linux" . $config['suffix']);
}

function set_os($config, $name, $os)
{
    if (! groupexist($config, $os)) {
        groupadd($config, $os, $config['parcs_rdn'], "parc des machines " . $os . $config['suffix']);
    }
    groupaddmember($config, $name . "$", $os . $config['suffix']);
}

// Delegations -------------------------------------------------------------------
/**
 *
 * @Description teste si l'utilisateur (loggé) a les droits sur le parc
 * @parametres $config, $parc, $right
 * @type
 * @returns true ou false
 */
function have_delegation($config, $parc, $right, $name = "login")
{
    if (type_delegation($config, $parc, $name) == $right)
        return true;
    return false;
}

/**
 *
 * @Description recherche les parcs délégués à l'utilisateur ou groupe, recursive sur les groupes d'appartenance
 * @parametres config, $name = "login", $recurse = true
 * @type
 * @returns array(user, parc, level) ou false
 */
function list_delegations($config, $name = "login", $recurse = true, $level = "")
{
    $parc = array();
    if ($name == "login")
        $name = $config['login'] ?? "";
    if (! empty($name)) {
        $user = search_user($config, $name);
        if (empty($user)) {
            $user = search_group($config, $name);
        }
        $cn = $user['cn'] ?? "";
        if (! empty($cn)) {
            if ($recurse) {
                $ret = list_delegations($config, $name, false, $level);
                // $user = search_user($config, $name);
                foreach ($user['memberof'] as $groupdn) {
                    $ret1 = list_delegations($config, ldap_dn2cn($groupdn), false, $level);
                    $ret = array_merge($ret1, $ret);
                }
                return $ret;
            } else {

                $res = search_ad($config, $name, "delegation");
                if ($res) {
                    foreach ($res as $key => $group) {
                        $delegation = explode('_', $group['cn'], 2);
                        if ((! empty($level) && $delegation[0] == $level) || empty($level)) {
                            $parc[$key]['user'] = $cn;
                            $parc[$key]['cn'] = $delegation[1];
                            $parc[$key]['level'] = $delegation[0];
                            $parc[$key]['samaccountname'] = $group['samaccountname'];
                            $parc[$key]['dn'] = $group['dn'];
                        }
                    }
                    return $parc;
                }
            }
        }
    }
    return array();
}

/**
 * niveau de Délégation de parc pour un user
 *
 * @Description retourne le niveau de délégation de l'utlisateur courant pour une machine ou un parc
 * @parametre : $config, $name nom du parc ou de la machine, $user = "login"
 * @returns : string "manage", "view", "rdp", false
 */
function type_delegation($config, $name, $user = "login")
{
    $parcs = list_delegations($config, $user);
    $ret = false;
    if ($parcs) {
        foreach ($parcs as $parc) {
            if ($parc['cn'] == $name)
                return $parc['level'];
            if (is_member_parc($config, $parc['cn'], $name)) {
                if ($parc['level'] == "manage") {
                    return "manage";
                } elseif ($parc['level'] == "rdp") {
                    return "rdp";
                } elseif ($parc['level'] == "vnc") {
                    return "vnc";
                } else {
                    $ret = "view";
                }
            }
        }
    }
    return $ret;
}

/**
 * delegations en cours pour un parc
 *
 * @Description cherche les delegations pour un parc
 * @parametres : array $config, string $parc
 * @returns : array
 *
 */
function search_delegations(array $config, string $parc = "")
{
    if (empty($parc)) {
        $filter = "*";
    } else {
        $filter = "*_" . $parc;
    }
    $res = search_ad($config, $filter, "group", $config['dn']['delegations']);
    foreach ($res as $key => $delegation) {
        $del = explode("_", $delegation['cn'], 2);
        $res[$key]['level'] = $del[0];
        $res[$key]['parc'] = $del[1];
    }
    return $res;
}

/**
 * Mise en place délégation
 *
 * @Description met en place une delegation pour un parc
 * @parametres : $config, $parc, $name, $level)
 * @returns : array
 *
 */
function create_delegation($config, $parc, $name, $level)
{
    $delegation = $level . "_" . $parc;
    $res = search_ad($config, $delegation, "group", $config['dn']['delegations']);
    $suffix = $config['suffix'];
    $ret = true;
    if (! $res) {
        $ret = groupadd($config, $delegation, $config['delegations_rdn'], "délégation du droit " . $level, $suffix);
    }
    if ($ret)
        $ret = groupaddmember($config, $name, $delegation . $suffix);
    return $ret;
}

/**
 *
 * @description supprime une delegation pour un utilisateur ou groupe
 *
 */
function delete_delegation($config, $parc, $name, $level)
{
    $delegation = $level . "_" . $parc;
    $delegation = $parc;
    $res = search_ad($config, $delegation, "group", $config['dn']['delegations']);
    $ret = groupdelmember($config, $name, $res[0]['samaccountname']);
    if (count($res[0]['member']) == 0) {
        $ret = groupdel($config, $delegation);
    }
    return $ret;
}

// ----------------------------------------------------
// dhcp
/**
 * retourne l'ip fixe d'un serveur ad ou fs secondaire a partir du réseau actuel.
 *
 * @param unknown $config
 * @param unknown $ip
 * @param unknown $type
 * @return string
 */
function slave_ip($config, $ip, $type)
{
    $net = explode(".", $ip);
    $m = explode(".", $config[$type . '_ip']);
    if ($m[2] == $net[2] && $type = "se4ad") {
        // on veut un ad secondaire sur le même réseau
        $slave = implode(".", [
            $m[0],
            $m[1],
            $m[2],
            $m[3] + 2
        ]);
    } else {
        // on est en multi etab
        $slave = implode(".", [
            $m[0],
            $m[1],
            $net[2],
            $m[3]
        ]);
    }
    return $slave;
}

/**
 * retourne la reservation, false dans les autres cas
 *
 * @param unknown $config
 * @param unknown $machine
 *            nom, ip ou mac
 * @return unknown[]|boolean
 */
function get_dhcp_reservation($config, $machine)
{
    $res = search_ad($config, $machine, "machine");
    if (isset($res[0]['iphostnumber']) && isset($res[0]['networkaddress']) && $res[0]['state'] == "reservation") { // != "dns" && $res[0]['state'] != "reservation") {
        return array(
            'cn' => $res[0]['cn'],
            'iphostnumber' => $res[0]['iphostnumber'],
            'networkaddress' => $res[0]['networkaddress']
            // 'state' => $res[0]['state']
        );
    } else {
        return false;
    }
}

/**
 * renvoie l'ip si elle est libre ou une ip fixe libre dans le meme vlan
 *
 * @Parametres $ip : Adresse IP a tester
 * @return adresse ip libre
 *        
 */
function get_free_ip($config, $ip, &$html)
{
    $reseau = get_vlan($config, $ip);
    $calcul_ip = floatval(sprintf("%u", ip2long($ip)));
    $ipmin = floatval(sprintf("%u", $reseau['ip_min']));
    $ipmax = floatval(sprintf("%u", $reseau['ip_max']));
    $gateway = floatval(sprintf("%u", $reseau['gateway']));
    $begin_range = floatval(sprintf("%u", $reseau['begin_range']));
    $end_range = floatval(sprintf("%u", $reseau['end_range']));

    if ((isset(get_dhcp_reservation($config, $ip)['iphostnumber'])) or ($ip == long2ip($reseau['gateway'])) or ($calcul_ip < $ipmin) or (($calcul_ip >= $begin_range) and ($calcul_ip <= $end_range))) {
        $calcul_free = $reseau['ip_min'];
        while (floatval(sprintf("%u", $calcul_free)) <= $ipmax) {
            if (ping(long2ip($calcul_free)) or (isset(get_dhcp_reservation($config, long2ip($calcul_free))['iphostnumber']) or ($calcul_free == $reseau['gateway']))) {
                // echo long2ip($calcul_free) . " prise<br>";
                $calcul_free ++;
            } elseif ($calcul_free == $reseau['begin_range']) {
                $calcul_free = $reseau['end_range'] + 1;
            } else {
                $html .= "l'adresse <b>" . long2ip($calcul_free) . "</b> est disponible<br>";
                return long2ip($calcul_free);
            }
        }
        return false;
    } else {
        $html .= "l'adresse <b>" . $ip . " </b>saisie est libre<br>";
        return $ip;
    }
}

/**
 *
 * @description enregistre une reservation dhcp dans AD
 *  l'ip est basée sur le bail actuel de la machine
 *  la machine doit déjà avoir un enregistrement ad
 *  si une ip est passée on tente de la reserver.
 * @parametres : $config, $cn
 * @returns : true si ok
 */
function set_dhcp_reservation($config, $cn, $ip = "", $mac = "", &$html = "")
{
    $dhcp = true;
    $mode = false;
    // search_machine a besoin de l'argument N3 true pour fournir l'index state
    $machine = search_machine($config, $cn, true);
    if (count($machine) > 0) {
        $state = $machine['state'] ?? "";
        // if (! empty($machine['networkaddress']) && (! empty($machine['iphostnumber']) || $ip)) {
        if ($state == "reservation") {
            // if (! empty($ip) && $ip != $machine['iphostnumber']) {
            if (! empty($ip) && $ip != $machine['iphostnumber'] || ! empty($ip) && $ip == $machine['iphostnumber']) {
                $mode = "replace";
            }
        } elseif ((! empty($mac) || ! empty($machine['networkaddress'])) && (! empty($machine['iphostnumber']) || ! empty($ip))) {
            $reservation = get_dhcp_reservation($config, $machine['networkaddress'] ?? $mac);
            if (isset($reservation['iphostnumber'])) {
                if ($machine['iphostnumber'] != $reservation['iphostnumber']) {

                    // un enregistrement existe sous un autre nom pour cette ip
                    // @TODO DANGER !!!
                    // on ne supprime les machines que si elle ont une reservation identique
                    $old = search_machine($config, $reservation['iphostnumber']);
                    if (count($old) > 0) {
                        // if ($old['cn'] != $machine['cn'] && @$old['state'] == 'reservation') {
                        if ($old['netbootguid'] != $machine['netbootguid'] && @$old['state'] == 'reservation') {
                            delete_ad($config, $old['cn'], 'machine');
                            $mode = "add";
                        }
                    }
                }
            }
            $mode = "add";
        } elseif ((empty($mac) && empty($machine['networkaddress'])) && (empty($machine['iphostnumber']) || ! empty($ip))) {
            $mode = "add";
            $dhcp = false;
        }
        if (empty($ip)) {
            $ip = $machine['iphostnumber'] ?? "";
        }
        if ($dhcp) {
            $ip = get_free_ip($config, $ip, $html);
            $attrs = array(
                'networkaddress' => $machine['networkaddress'] ?? $mac
            );
            modify_ad($config, $machine['cn'], "machine", $attrs, $mode);
            unset($attrs);
        }
        $attrs = array(
            'iphostnumber' => $ip
        );
        if (! $mode) {
            return true;
        } else {
            return modify_ad($config, $machine['cn'], "machine", $attrs, $mode);
            // @TODO
            // ajouter la mise à jour du reservations.inc et restart dhcpd
        }
    }
    return false;
}

/**
 * enregistre une reservation dhcp dans AD
 *
 * @parametres : $machine
 * @parametres : $reservation - tableau {ip, mac}
 * @returns : true si ok
 */
function delete_dhcp_reservation($config, $machine)
{
    $reservation = get_dhcp_reservation($config, $machine);
    if (isset($reservation['iphostnumber'])) {
        $del['iphostnumber'] = $reservation['iphostnumber'];
        return modify_ad($config, $machine, "machine", $del, "del");
    } else {
        return false;
    }
}

function delete_all_dhcp_reservations($config)
{
    foreach (search_ad($config, "*", "machine") as $machine) {
        if (isset($machine['iphostnumber']) && $machine['state'] == "reservation") {
            modify_ad($config, $machine['cn'], "machine", [
                'iphostnumber' => $machine['iphostnumber']
            ], "del");
        }
    }
}

/**
 * Importe le fichier des reservations /etc/sambaedu/reservations.inc vers l'AD
 *
 * @returns : array machine, ip, mac
 */
function import_dhcp_reservations($config)
{
    if (file_exists("/etc/sambaedu/reservations.inc.se3")) {
        $contents = file_get_contents("/etc/sambaedu/reservations.inc.se3");
        $contents = explode("\n", $contents);
        $index = 0;
        $data = array();
        $m = array();
        $html = "";
        $record = false;
        foreach ($contents as $line) {
            if (preg_match("/^\s*(|#.*)$/", $line, $m)) {
                // on saute les commentaires
            } elseif (preg_match("/^host (.*)$/", $line, $m) && (! $record)) {
                $index ++;
                $data[$index]['cn'] = strtolower($m[1]);
            } elseif (preg_match("/^{/", $line)) {
                $record = true;
            } elseif (preg_match("/^\s*hardware ethernet\s*([0-9a-fA-F:]*)\s*;$/", $line, $m)) {
                $data[$index]['networkaddress'] = strtolower($m[1]);
            } elseif (preg_match("/^\s*fixed-address\s*([0-9\.]*)\s*;$/", $line, $m)) {
                $data[$index]['iphostnumber'] = $m[1];
            } elseif (preg_match("/}*/", $line, $m)) {
                $record = false;
            } else {
                $html .= "Erreur ligne '$line'\n";
                break;
            }
        }
        foreach ($data as $reservation) {
            if (! search_ad($config, $reservation['cn'], "machine")) {
                create_machine($config, $reservation['cn'], $config['equipements_rdn']);
            }
            $res = set_dhcp_reservation($config, $reservation['cn'], $reservation['iphostnumber'], $reservation['networkaddress']);
            if ($res) {
                $html .= "machine " . $reservation['cn'] . ": ip " . $reservation['iphostnumber'] . " OK\n";
            } else {
                $html .= "machine " . $reservation['cn'] . ": ip " . $reservation['iphostnumber'] . " ERREUR\n";
            }
        }
    }
    return $html;
}

function import_computer_list($config, $csv_file, $file_type = "csv_machines", &$data = [])
{
    $contents = file_get_contents($csv_file);
    if ($file_type == "csv_machines") {
        $contents = explode("\n", $contents);
        foreach ($contents as $num => $line) {
            $line = rtrim($line);
            if ($num == 0) {
                if (preg_match("/name/", $line)) {
                    $keys = preg_replace("/\"/", "", explode(";", $line, 80));
                } else {
                    $keys = [
                        'name',
                        'ip',
                        'mac',
                        'type',
                        'ou',
                        'prise',
                        'etab_uai'
                    ];
                }
            } elseif (! empty($line)) {
                $index = $num - 1;
                $val = preg_replace("/\"/", "", explode(";", $line, 80));
                if (count($val) > 0) {
                    foreach ($keys as $ind => $key) {
                        if (! empty($key)) {
                            $data[$index][$key] = $val[$ind] ?? "";
                        }
                    }
                }
            }
        }
    }
    $n = $index + 1;
    if ($n == 1) {
        $n = "aucun";
    }

    $html = "Fichier " . $csv_file . " format : " . $file_type . "  : " . $n . " machines analysées<br>";
    $html .= "<pre>" . print_r($data, true) . "</pre>";
    return $html;
}

function import_site_list($config, $csv_file, $file_type = "csv_sites", &$data = [])
{
    $contents = file_get_contents($csv_file);
    if ($file_type == "csv_sites") {
        $contents = explode("\n", $contents);
        foreach ($contents as $num => $line) {
            $index = - 1;
            $line = rtrim($line);
            if ($num == 0) {
                if (preg_match("/UAI/", $line)) {
                    $keys = preg_replace("/\"/", "", explode(";", $line, 80));
                }
            } elseif (! empty($line)) {
                $index = $num - 1;
                $val = preg_replace("/\"/", "", explode(";", $line, 80));
                if (count($val) > 0) {
                    foreach ($keys as $ind => $key) {
                        if (! empty($key)) {
                            $data[$index][$key] = strtolower($val[$ind]) ?? "";
                        }
                    }
                }
            }
        }
    }
    $n = $index + 1;
    if ($n == 0) {
        $n = "aucun";
    }

    $html = "Fichier " . $csv_file . " format : " . $file_type . "  : " . $n . " sites analysés<br>";
    $html .= "<pre>" . print_r($data, true) . "</pre>";
    return $html;
}

function update_sites($config, $duree)
{
    $central = empty($config['etab_ou']);
    $sites = apcu_fetch('ent');
    $html = apcu_fetch('sites_html');
    $res = $size = $nb = 0;
    if (is_array($sites)) {
        $res = $size = count($sites);
    }
    $time = time();
    while ($res > 0 && $nb ++ < $size && (time() - $time < $duree)) {

        $site = array_shift($sites);
        $res = $size = count($sites);
        $html .= "site " . $site['UAI'] . " :<br>\n";
        $ret = true;
        $uai = $site['UAI'];
        $config = get_config($config, false);
        if (! search_ad($config, $site['UAI'], "site")) {
            if ($ret = create_site($config, $site['UAI'])) {
                $uai = $site['UAI'];
                unset($site['UAI']);
                $ret &= create_site_link($config, $uai, $uai);
                foreach ($site as $k => $val) {
                    switch ($k) {
                        case "Pedagogie":
                            $sub = "/22";
                            $vlan = 3;
                            break;
                        case "Perdir":
                            $sub = "/24";
                            $vlan = 2;
                            break;
                        case "Imprimantes":
                            $sub = "/24";
                            $vlan = 101;
                            break;
                        case "Serveurs":
                            $sub = "/24";
                            $vlan = 100;
                            break;
                        Case "PMX":
                            $sub = "/24";
                            $vlan = 253;
                            break;
                    }
                    $ret &= create_site_subnet($config, $val . $sub, $uai, $k, $vlan);
                }
            }
        }
        if ($ret) {
            $html .= " création site $uai OK, \n";
            trigger_error("création site $uai OK");
        } else {
            $html .= " création site $uai  ERREUR<br>\n";
            trigger_error("création site $uai ERREUR");
        }
    }
    if ($res > 0) {
        $end = false;
    } else {
        $end = true;
        $html .= "<b>Import terminé !</b>\n";
    }
    apcu_store('ent', $sites, 600);
    apcu_store('sites_count', $res, 600);
    apcu_store('sites_html', $html);

    return $end;
}

function update_computers($config, $duree)
{
    $central = empty($config['etab_ou']);
    $machines = apcu_fetch('ent');
    $html = apcu_fetch('machines_html');
    $res = $size = $nb = 0;
    if (is_array($machines)) {
        $res = $size = count($machines);
    }
    $time = time();
    while ($res > 0 && $nb ++ < $size && (time() - $time < $duree)) {

        $reservation = array_shift($machines);
        $res = $size = count($machines);
        $html .= "machine " . $reservation['name'] . ", Salle " . $reservation['ou'] . " :<br>\n";
        $ret = true;
        $config = get_config($config, false);
        if (! search_ad($config, $reservation['name'], "machine")) {
            if ($reservation['type'] == "equipement") {
                if (! empty($reservation['etab_uai']) && $central) {
                    $ou = "OU=" . $reservation['etab_uai'] . "," . $config['equipements_rdn'];
                } else {
                    $ou = $config['equipements_rdn'];
                }
            } else {
                if (! empty($reservation['etab_uai']) && $central) {
                    $ou = "OU=" . $reservation['etab_uai'] . "," . $config['computers_rdn'];
                } else {
                    $ou = $config['computers_rdn'];
                }
            }
            if (! empty($reservation['ou'])) {
                $exists = false;
                $salles = search_parcs($config, $reservation['ou'], "salle");
                foreach ($salles as $salle) {
                    if (! empty($salle['dn']) && ! empty($reservation['etab_uai']) && preg_match("/" . $reservation['etab_uai'] . "/i", $salle['dn'])) {
                        $ou = preg_replace("/," . $config['ldap_base_dn'] . "/i", "", $salle['dn']);
                        $exists = true;
                        break;
                    }
                }
                if (! $exists) {
                    if (! empty($reservation['etab_uai']) && $central) {
                        $parent = $reservation['etab_uai'];
                        $config['suffix'] = "-" . substr($reservation['etab_uai'], 3);
                        $config['parcs_rdn'] = "OU=" . $reservation['etab_uai'] . "," . $config['parcs_rdn'];
                    } else {
                        $parent = "";
                    }
                    $ret = create_parc($config, $reservation['ou'], "", "salle", $parent);
                    $ou = "OU=" . $reservation['ou'] . "," . $ou;
                }
            }
            if ($ret) {
                $ret &= create_machine($config, $reservation['name'], $ou, "Import fichier");
            }
        }
        if ($ret) {
            $html .= " création OK, \n";
        } else {
            $html .= " création ERREUR<br>\n";
            $html .= "ou : " . $ou . " nom : " . $reservation['name'] . "<br>";
        }

        if ($ret) {
            if (empty($reservation['ip']) || $central) {
                $attrs = array(
                    'networkaddress' => $reservation['mac']
                );
                $mode = "add";
                $ret = modify_ad($config, $reservation['name'], "machine", $attrs, $mode);
                if ($ret) {
                    $html .= " ip dynamique, mac " . $reservation['mac'] . " OK<br>\n";
                } else {
                    $html .= " mac " . $reservation['mac'] . " ERREUR<br>\n";
                }
            } else {
                $ret = set_dhcp_reservation($config, $reservation['name'], $reservation['ip'], $reservation['mac'], $html);
                if ($ret) {
                    $html .= " reservation " . $reservation['ip'] . " OK<br>\n";
                } else {
                    $html .= " reservation " . $reservation['ip'] . " ERREUR<br>\n";
                }
            }
        }
    }
    if ($res > 0) {
        $end = false;
    } else {
        $end = true;
        $html .= "<b>Import terminé !</b>\n";
    }
    apcu_store('ent', $machines, 600);
    apcu_store('machines_count', $res, 600);
    apcu_store('machines_html', $html);

    return $end;
}

function export_dhcp_reservations($config)
{
    $reservations = "/etc/sambaedu/reservations.inc";
    $content = "# reservations exportees automatiquement de l'annuaire AD\n";
    $content .= "# le " . date(DATE_RSS) . "\n";
    $machines = search_ad($config, "*", "machine_fast", "all");
    foreach ($machines as $machine) {
        if (! empty($machine['networkaddress']) && ! valid_mac($machine['networkaddress'])) {
            if (delete_dhcp_reservation($config, $machine['cn'])) {
                unset($machine['networkaddress']);
                echo "reservation de " . $machine['cn'] . " supprimée<br>\n";
            } else {
                echo "impossible de supprimer la reservation de " . $machine['cn'] . "<br>\n";
            }
        }
        $res = (! empty($machine['networkaddress']) && ! empty($machine['iphostnumber'])); // && preg_match("/^(25[0–5]|2[0–4][0–9]|[01]?[0–9][0–9]?).(25[0–5]|2[0–4][0–9]|[01]?[0–9][0–9]?).(25[0–5]|2[0–4][0–9]|[01]?[0–9][0–9]?).(25[0–5]|2[0–4][0–9]|[01]?[0–9][0–9]?)$/", $machine['iphostnumber'])));
        if ($res) {
            $content .= "host " . preg_replace("/ /", "-", $machine['cn']) . "\n";
            $content .= "{\n";
            $content .= "hardware ethernet " . $machine['networkaddress'] . ";\n";
            $content .= "fixed-address " . $machine['iphostnumber'] . ";\n";
            $content .= "}\n";
        }
    }
    if (! $handle = fopen($reservations, "w")) {
        die("Erreur d'ecriture des reservtions se4 : $reservations");
    }
    $res = fwrite($handle, $content);
    fclose($handle);
    return true;
}

/**
 * importe les leases dhcp.
 *
 * @Description Seul le dernier enregistrement actif d'une ip est conservé.
 * @type
 * @parametres array $config
 * @returns array(array(ip, name, mac, state))
 */
function import_dhcp_leases($config)
{
    $contents = file_get_contents("/var/lib/dhcp/dhcpd.leases");
    $contents = explode("\n", $contents);
    $current = 0;
    $data = $m = array();
    foreach ($contents as $line) {
        switch ($current) {
            case 0:
                if (preg_match("/^\s*(|autho.*|#.*)$/", $line, $m)) {} else if (preg_match("/^lease (.*) {/", $line, $m)) {
                    $current = $m[1];
                } else if (preg_match("/^server-duid/", $line)) {
                    // ignore
                } else {
                    print "Failed parsing '$line'\n";
                }
                break;
            default:
                if (preg_match('/^\s*(binding state|hardware ethernet|client-hostname|uid|set vendor-class-identifier =) "?(.*?)"?;$/', $line, $m)) {
                    $data[$current][$m[1]] = $m[2];
                } elseif (preg_match("/}/", $line, $m)) {
                    $current = 0;
                } else {
                    // print "Failed parsing '$line'\n";
                }
        }
    }
    $ret = array();
    foreach ($data as $ip => $d) {
        if ((($d['binding state'] == "active") || ($d['binding state'] == "free")) && (isset($d['hardware ethernet']) && (isset($d['client-hostname']) && (! empty($d['client-hostname'])) || (isset($d['uid']))))) {
            $ret[] = array(
                'name' => strtolower($d['client-hostname'] ?? "ipxe"),
                'ip' => $ip,
                'mac' => strtolower($d['hardware ethernet'] ?? ""),
                'state' => $d['binding state']
            );
        }
    }
    // apcu_store('dhcp_leases', $ret, 300);
    return $ret;
}

/**
 * liste les baux dhcp actifs
 *
 * @param unknown $config
 * @return unknown|unknown[][]|NULL[][]|mixed[][]
 */
function list_dhcp_leases($config)
{
    $sup = 0;
    $parser = import_dhcp_leases($config);
    $machines = search_ad($config, "*", "machine");
    foreach ($parser as $key => $lease) {
        foreach ($machines as $machine) {
            if (isset($machine['state']) && (isset($machine['networkaddress']) && $machine['networkaddress'] == $lease['mac'])) {
                if (($machine['state'] == "reservation") && ($machine['networkaddress'] == $lease['mac'])) {
                    unset($parser[$key]);
                    $sup ++;
                    break;
                } else {
                    $parser[$key]['state'] = $machine['state'];
                }
            }
        }
    }
    return $parser;
}

function is_printer($config, $machine)
{
    return (strtolower(ldap_dn2oudn($machine['dn'])) == strtolower($config['dn']['equipements']));
}

function search_dhcp_leases($config, $ip)
{
    $res = [];
    $machines = search_ad($config, "*", "machine");
    foreach ($machines as $machine) {
        if (preg_match("/" . $ip . "/", @$machine['iphostnumber']) || preg_match("/" . $ip . "/", @$machine['cn']) || preg_match("/" . $ip . "/", @$machine['networkaddress'])) {
            $res[] = $machine;
        }
    }
    return $res;
}

function get_dhcp_lease($config, $leases, $name)
{
    foreach ($leases as $data) {
        if (($data['ip'] == $name) || ($data['name'] == strtolower($name)) || ($data['mac'] == $name))
            return $data;
    }
    return false;
}

// ----------- gestion des classes------------------------------
function is_eleve($config, $name)
{
    if (is_array($name)) {
        foreach ($name['memberof'] as $g) {
            if (preg_match("/Eleves/", $g)) {
                return true;
            }
        }
        return false;
    } elseif (! empty($name)) {
        if (count(search_ad($config, $name, "member", "Eleves")) > 0)
            return true;
        else
            return false;
    }
}

function is_prof($config, $name)
{
    if (is_array($name)) {
        foreach ($name['memberof'] as $g) {
            if (preg_match("/Profs/", $g)) {
                return true;
            }
        }
        return false;
    } elseif (! empty($name)) {
        if (count(search_ad($config, $name, "member", "Profs")) > 0)
            return true;
        else
            return false;
    } else {
        return false;
    }
}

function is_pp(array $config, string $name)
{}

function list_groups($config, $user)
{
    $groups = [];
    $config_full = get_config($config, true, true);
    foreach (search_ad($config, $user, "user", $config['dn']["people"])['memberof'] as $dn) {
        $groups[] = search_group($config_full, $dn);
    }
    return $groups;
}

/**
 *
 * @description liste les classes pour une classe, un eleve, un prof, ou tous
 * @parametres array $config, string $name : eleve, prof, "*"
 * @return array $classe : le nom court des classes
 */
function list_classes(array $config, $name)
{
    $ret = array();
    if (is_array($name)) {
        $m = [];
        foreach ($name['memberof'] as $group) {
            if (preg_match("/CN=(Classe|Equipe)_(.*)," . $config['dn']['classes'] . "/i", $group, $m)) {
                $ret[] = $m[2];
            }
        }
    } else {

        $classes = search_ad($config, $name, "classe");
        if (! is_array($classes) || count($classes) == 0) {
            $classes = search_ad($config, $name, "equipe");
        }
        if (is_array($classes)) {
            /*
             * foreach ($classes as $val) {
             * $res = type_group($config, $val['cn']);
             * $ret[] = $res['nom'];
             * }
             */
            foreach ($classes as $classe) {
                $ret[] = preg_split("/_/", $classe['cn'], 2)[1];
            }
        }
    }
    return $ret;
}

/**
 *
 * @description liste les cours pour un cours, un eleve, un prof, ou tous
 * @parametres array $config, string $name : eleve, prof, "*"
 * @return array $classe : le nom court des cours
 */
function list_cours(array $config, $name)
{
    $ret = array();
    if (is_array($name)) {
        foreach ($name['memberof'] as $group) {
            if (preg_match("/CN=(Cours|Equipe)_(.*)," . $config['dn']['cours'] . "/i", $group, $m)) {
                $ret[] = $m[2];
            }
        }
    } else {

        $classes = search_ad($config, $name, "cours");
        if (count($classes) == 0) {
            $classes = search_ad($config, $name, "equipe");
        }
        if (! is_array($classes) || is_array($classes)) {
            /*
             * foreach ($classes as $val) {
             * $res = type_group($config, $val['cn']);
             * $ret[] = $res['nom'];
             * }
             */
            foreach ($classes as $classe) {
                $ret[] = preg_split("/_/", $classe['cn'], 2)[1];
            }
        }
    }
    return $ret;
}

/**
 *
 * @description Efface les groupes Equipe Classe et Cours vides
 * recherche limitée à l'établissement
 * @parametres array $config, string &$html
 * @return bool $res
 */
function clean_groups(array $config, string &$html)
{
    $html .= "<b>Nettoyage des éventuels groupes vides :</b><br>\n";
    $res = true;
    // recherche locale sur l'étab
    $equipes = search_ad($config, "*", "equipe");
    $classes = search_ad($config, "*", "classe");
    $cours = search_ad($config, "*", "cours");
    $n = 0;
    foreach ($equipes as $equipe) {
        if (! isset($equipe['member']) || count($equipe['member']) == 0) {
            $name = preg_replace("/Equipe_/", "", $equipe['cn']);
            $e = true;
            if ($k = array_search("Classe_" . $name, array_column($classes, 'cn'))) {
                if (! isset($classes[$k]['member']) || count($classes[$k]['member']) == 0) {
                    $res &= groupdel($config, "Classe_" . $name . $config['suffix']);
                    $res &= groupdel($config, "Equipe_" . $name . $config['suffix']);
                    $res &= groupdel($config, "PP_" . $name . $config['suffix']);
                    $html .= "Suppression de la classe " . $name . "<br>\n";
                    $e = true;
                } else {
                    $e = false;
                }
            }
            if ($k = array_search("Cours_" . $name, array_column($cours, 'cn'))) {
                if (! isset($cours[$k]['member']) || count($cours[$k]['member']) == 0) {
                    $res &= groupdel($config, "Cours_" . $name . $config['suffix']);
                    $res &= groupdel($config, "Equipe_" . $name . $config['suffix']);
                    $html .= "Suppression du cours " . $name . "<br>\n";
                    $e = true;
                } else {
                    $e = false;
                }
            }
            if ($e) {
                $res &= groupdel($config, "Equipe_" . $name . $config['suffix']);
                $html .= "Suppression de l'équipe " . $name . "<br>\n";
                $n ++;
            }
        }
    }
    $equipes = search_ad($config, "*", "equipe");
    $classes = search_ad($config, "*", "classe");
    foreach ($classes as $classe) {
        if (! isset($classe['member']) || count($classe['member']) == 0) {
            $name = preg_replace("/Classe_/", "", $classe['cn']);
            $e = true;
            if ($k = array_search("Equipe_" . $name, array_column($equipes, 'cn'))) {
                if (! isset($equipes[$k]['member']) || count($equipes[$k]['member']) == 0) {
                    $res &= groupdel($config, "Classe_" . $name . $config['suffix']);
                    $res &= groupdel($config, "Equipe_" . $name . $config['suffix']);
                    $res &= groupdel($config, "PP_" . $name . $config['suffix']);
                    $html .= "Suppression de la classe " . $name . "<br>\n";
                    $e = true;
                } else {
                    $e = false;
                }
            }
            if ($e) {
                $res &= groupdel($config, "Classe_" . $name . $config['suffix']);
                $html .= "Suppression de la classe " . $name . "<br>\n";
                $n ++;
            }
        }
    }
    $equipes = search_ad($config, "*", "equipe");
    $cours = search_ad($config, "*", "cours");
    foreach ($cours as $c) {
        if (! isset($c['member']) || count($c['member']) == 0) {
            $name = preg_replace("/Cours_/", "", $c['cn']);
            $e = true;
            if ($k = array_search("Equipe_" . $name, array_column($equipes, 'cn'))) {
                if (! isset($equipes[$k]['member']) || count($equipes[$k]['member']) == 0) {
                    $res &= groupdel($config, "Cours_" . $name . $config['suffix']);
                    $res &= groupdel($config, "Equipe_" . $name . $config['suffix']);
                    $html .= "Suppression du cours " . $name . "<br>\n";
                    $e = true;
                } else {
                    $e = false;
                }
            }
            if ($e) {
                $res &= groupdel($config, "Cours_" . $name . $config['suffix']);
                $html .= "Suppression du cours " . $name . "<br>\n";
                $n ++;
            }
        }
    }
    if ($n > 0) {
        $html .= "Suppression de $n groupes.<br>\n";
    } else {
        $html .= "Pas de groupes vides à supprimer.<br>\n";
    }
    return $res;
}

/**
 *
 * @description liste les profs pour une classe, un eleve ou tous
 * @type
 * @parametres array $config, string $name : classe ou eleve ou "*"
 * @returns array [profs]
 */
function list_profs($config, $name)
{
    $classes = list_classes($config, $name);
    $profs = array();
    foreach ($classes as $classe) {
        $equipe = search_ad($config, $classe, "equipe");
        // une seule equipe ?
        if (isset($equipe[0]['member'])) {
            foreach ($equipe[0]['member'] as $prof) {
                $profs[] = $prof;
            }
        }
    }
    return $profs;
}

/**
 *
 * @description retourne la liste des eleves du prof, ou de la classe ou tous
 * @type
 * @parametres array $config, string $name : prof, classe, "*"
 * @returns array [eleves]
 */
function list_eleves($config, $name)
{
    $classes = list_classes($config, $name);
    $eleves = array();
    foreach ($classes as $classe) {
        $res = search_ad($config, $classe, "classe");
        // une seul resultat ?
        if (isset($res[0]['member'])) {
            foreach ($res[0]['member'] as $eleve) {
                $eleves[] = $eleve;
            }
        }
    }
    return $eleves;
}

/**
 *
 * @description retourne la liste des profs principaux de l'eleve, de la classe ou tous
 * @type
 * @parametres array $config, string $name : eleve, classe, "*"
 * @returns array(pp)
 */
function list_pp($config, $name)
{
    $classes = list_classes($config, $name);
    $pps = array();
    foreach ($classes as $classe) {
        $res = search_ad($config, $classe, "pp");
        // une seule equipe ?
        if (isset($res[0]['member'])) {
            foreach ($res[0]['member'] as $pp) {
                $pps[] = $pp;
            }
        }
    }
    return $pps;
}

/**
 * prof de l'élève ?
 *
 * @description teste si $prof est prof pour $name (eleve ou classe)
 * @type
 * @parametres array $config, string $name eleve ou classe, string $prof : prof
 * @returns boolean
 */
function is_my_prof($config, $name, $prof)
{
    $profs = list_profs($config, $name);
    $dn = search_user($config, $prof);
    return in_array($dn['dn'], $profs);
}

/**
 * est-il mon élève ?
 *
 * @description teste si $eleve est eleve pour $name (prof ou classe)
 * @type
 * @parametres array $config, string $name: prof ou classe, string $eleve: eleve
 * @returns boolean
 */
function is_my_eleve($config, $name, $eleve)
{
    $eleves = list_eleves($config, $name);
    $dn = search_user($config, $eleve);
    return (in_array($dn['dn'], $eleves) && ($name != $eleve));
}

/**
 * Test Prof principal
 *
 * @description teste si $prof est prof principal pour $name (eleve ou classe)
 *
 * @param array $config
 * @param string $name
 *            eleve, classe
 * @param string $prof
 *            prof
 * @return boolean
 */
function is_my_pp($config, $eleve, $prof)
{
    $pps = list_pp($config, $eleve);
    $dn = search_user($config, $prof);
    if (isset($dn['dn']))
        return in_array($dn['dn'], $pps);
    else
        return false;
}

/**
 * Type de groupe
 *
 * @Description retourne le type de groupe
 * @parametres array $config, string $name : nom complet du groupe (Classe_TRUC) ou court (TRUC)
 * @returns array nom et type du groupe. si les conventions de nommage et de placement ne sont pas vérifiées , retourne type=>"groupe"
 * si le nom court est passé, rtourne "classe"
 */
function type_group(array $config, string $name)
{
    $groups = search_ad($config, $name, "group", $config['dn']['groups']);
    if (count($groups) > 0) {
        if (preg_match("/(Classe|Equipe|Cours)_(.*)-?([0-9]{4}[a-z])?$/U", $name, $m)) {
            return array(
                'nom' => $m[2],
                'type' => strtolower($m[1])
            );
        } else {
            foreach ($groups as $group) {
                $cn = explode("_", $group['cn'], 2);
                if (strcasecmp("Classe", $cn[0]) == 0) {
                    return array(
                        'nom' => $cn[1],
                        'type' => "classe"
                    );
                } elseif (strcasecmp("Cours", $cn[0]) == 0) {
                    return array(
                        'nom' => $cn[1],
                        'type' => "cours"
                    );
                }
            }

            return array(
                'nom' => $name,
                'type' => "groupe"
            );
        }
    }

    return false;
}

/**
 * Ajout d'un enseble de groupes pédagogiques dans l'AD
 *
 * @Description crée les groupe(s) classe+equipe+pp (type classe) ou cours+equipe (type cours) ou directement le nom (tyep groupe)
 * @parametres array $config, string $name : nom court sans le prefixe (Classe_, Equipe_) : nom du groupe + suffix eventuel
 * string $description, string $type : classe (classe+equipe), matiere, cours, projet, groupe
 * string type
 * @returns boolean
 */
function create_group(array $config, string $name, string $description, string $type = "groupe")
{
    $res = type_group($config, $name);
    if (is_array($res) && ($type != $res['type'])) {
        // on ne cree pas de groupe du meme nom qu'une classe !
        return false;
    }
    if (($type == "classe") || ($type == "equipe")) {
        $classe = "Classe_" . $name;
        $ou = $config['classes_rdn'] . "," . $config['groups_rdn'];
        // ouadd($config, $config['groups_rdn']);
        // ouadd($config, $ou . "," . $config['ldap_base_dn']);
        $res = groupadd($config, $classe, $ou, $description, $config['suffix']);
        $equipe = "Equipe_" . $name;
        $ou = $config['equipes_rdn'] . "," . $config['groups_rdn'];
        ouadd($config, $ou . "," . $config['ldap_base_dn']);
        $res &= groupadd($config, $equipe, $ou, "Equipe pédagogique de " . $description, $config['suffix']);
        $pp = "PP_" . $name;
        $ou = $config['equipes_rdn'] . "," . $config['groups_rdn'];
        $res &= groupadd($config, $pp, $ou, "Profs principaux de " . $description, $config['suffix']);
    } elseif ($type == "cours") {
        $cours = "Cours_" . $name;
        $ou = $config['cours_rdn'] . "," . $config['groups_rdn'];
        // ouadd($config, $config['groups_rdn']);
        // ouadd($config, $ou . "," . $config['ldap_base_dn']);
        $res = groupadd($config, $cours, $ou, "Cours de " . $description, $config['suffix']);
        $equipe = "Equipe_" . $name;
        $ou = $config['equipes_rdn'] . "," . $config['groups_rdn'];
        $res &= groupadd($config, $equipe, $ou, "Equipe pédagogique de " . $description, $config['suffix']);
        // if (! ouexist($config, $ouName[1], "Groups")) {
        // ouadd($config, $ouName[1], "Groups");
        // }
    } elseif ($type == "projet") {
        $projet = "Projet_" . $name;
        $ou = $config['projets_rdn'] . "," . $config['groups_rdn'];
        $res = groupadd($config, $projet, $ou, "Projet " . $description, $config['suffix']);
    } elseif ($type == "other_group") {
        $classe = ucfirst($name);
        $ou = $config[$type . "s_rdn"] . "," . $config['groups_rdn'];
        // On crée l'OU si elle n'existe pas
        $ouName = explode("=", $config[$type . "s_rdn"]);
        if (! ouexist($config, $ouName[1], $config['groups_rdn'])) {
            ouadd($config, $ouName[1], $config['groups_rdn']);
        }
        $res = groupadd($config, $classe, $ou, $description, $config['suffix']);
    } else {
        $classe = ucfirst($type) . "_" . $name;
        $ou = $config[$type . "s_rdn"] . "," . $config['groups_rdn'];
        $res = groupadd($config, $classe, $ou, $description, $config['suffix']);
    }
    return $res;
}

/**
 * ajoute un eleve au groupe
 *
 * crée dans le contexte d'étab défini par $config
 *
 * @param array $config
 * @param string $classe
 *            nom du groupe (forme courte admise)
 * @param array $user
 * @param string $type
 *            classe, cours, projet
 * @param bool $update
 *            : mettre à jour les partages (eleve) ou prof principal (prof)
 * @return boolean
 */
function add_user_group(array $config, string $classe, array $user, bool $update = false, bool $pp = false, $type = "")
{
    if (empty($type)) {
        $res = type_group($config, $classe);
        $type = $res['type'];
        $nom = $res['nom'];
    } else {
        $nom = $classe;
    }
    $type_user = type_user($config, $user);
    if ($type == "classe" || $type == "cours") {
        if ($type_user == "Eleves") {
            $res = groupaddmember($config, $user, ucfirst($type) . "_" . $nom . $config['suffix']);
            // un élève ne peut être que dans une classe !
            // NON ! c'est le tableau des diffs qui fait cela
            // if ($res)
            // if ($type == "classe") {
            // $oldclasses = list_classes($config, $user['cn']);
            // foreach ($oldclasses as $oldclasse) {
            // if ($oldclasse != $nom) {
            // $res &= groupdelmember($config, $user, ucfirst($type) . "_" . $oldclasse . $config['suffix']);
            // }
            // }
            // }
            if ($update) {
                $res &= update_eleve($config, $user);
            }
        } elseif ($type_user == "Profs") {
            $res = groupaddmember($config, $user, "Equipe_" . $nom . $config['suffix']);
            if ($res) {
                $my_pp = is_my_pp($config, $classe, $user['cn']);
                if ((! $my_pp) && $pp) {
                    $res = groupaddmember($config, $user, "PP_" . $nom . $config['suffix']);
                    $res = groupaddmember($config, $user, "Profs_Principaux", $config['suffix']);
                } elseif ($my_pp && (! $pp)) {
                    $res = groupdelmember($config, $user, "PP_" . $nom . $config['suffix']);
                    // si il ne reste plus aucune classe ou il est pp
                    if (count(list_pp($config, $user)) == 0) {
                        $res = groupdelmember($config, $user, "Profs_Principaux" . $config['suffix']);
                    }
                }
            }
        } else {
            // CPE, AED...
            $res = groupaddmember($config, $user, "Equipe_" . $nom . $config['suffix']);
        }
    } else {
        $classe = ucfirst($type) . "_" . $nom;
        $res = groupaddmember($config, $user, $classe . $config['suffix']);
    }
    return $res;
}

/**
 * nettoie l'appartenance classe/equipe d'un utilisateur en cas de besoin
 *
 * @param array $config
 * @param string $classe
 * @param string $eleve
 * @param bool $update
 * @return boolean
 */
function clean_user_group(array $config, string $classe, string $user, $type = "")
{
    if (empty($type)) {
        $res = type_group($config, $classe);
        $type = $res['type'];
        $nom = $res['nom'];
    } else {
        $nom = $classe;
    }
    if ($type == "classe") {
        if (is_eleve($config, $user)) {
            $res = groupdelmember($config, $user, "Equipe_" . $nom . $config['suffix']);
        } elseif (is_prof($config, $user)) {
            $res = groupdelmember($config, $user, "Classe_" . $nom . $config['suffix']);
        }
    } elseif ($type == "cours") {
        if (is_eleve($config, $user)) {
            $res = groupdelmember($config, $user, "Equipe_" . $nom . $config['suffix']);
        } else {
            $res = groupdelmember($config, $user, "Cours_" . $nom . $config['suffix']);
        }
    }
    return $res;
}

function delete_group($config, $name)
{
    $res = type_group($config, $name);

    if (is_array($res)) {
        $type = $res['type'];
        $classe = $res['nom'];
        if (($type == "classe") || ($type == "equipe")) {
            $name = "Classe_" . $classe;
            $res = groupdel($config, $name . $config['suffix']);
            $equipe = "Equipe_" . $classe;
            $res = groupdel($config, $equipe . $config['suffix']);
            $pp = "PP_" . $classe;
            $res = groupdel($config, $pp . $config['suffix']);
        } else {
            $res = groupdel($config, $name . $config['suffix']);
        }
    }
    return $res;
}

/*
 * test si $name (un cn) est prof principal de $classe
 * retourne un boolean
 */
function is_pp_this_classe($config, string $name, string $classe)
{
    $lespp = array();
    $lespp = list_pp($config, $classe);
    return preg_grep("/$name/i", $lespp);
}

/*
 * retourne un tableau [cn,Prenom Nom,Nom,sexe,Ddn, dn],
 * trié par Nom croissant
 * des membres du groupe $groupe
 */
function search_people_group($config, string $groupe)
{
    $res = search_ad($config, "*", "memberof", $groupe, [
        'dn',
        'pwdlastset'
    ]);
    usort($res, "cmp_nom");
    return $res;
}

/*
 * Retourne le tableau de la liste des classes
 * sous la forme de leur nom courtavec un cache
 * de 30 minutes
 * @return array
 */
function list_classes_etab_fast(array $config)
{
    $retour = array();
    $classes = search_ad($config, "Classe_*", "group", $config['dn']['groups']);
    foreach ($classes as $classe) {
        $retour[] = preg_replace("/Classe_/", "", $classe['cn']);
    }
    sort($retour);
    return $retour;
}

?>